@@ -61,8 +61,7 @@ rasqal_new_algebra_node(rasqal_query* query, rasqal_algebra_node_operator op)
   if(!query)
     return NULL;
   
-  node = (rasqal_algebra_node*)RASQAL_CALLOC(rasqal_algebra, 1, 
-                                             sizeof(rasqal_algebra_node));
+  node = RASQAL_CALLOC(rasqal_algebra_node*, 1, sizeof(*node));
   if(!node)
     return NULL;
 
@@ -268,8 +267,9 @@ rasqal_new_leftjoin_algebra_node(rasqal_query* query,
  * @query: #rasqal_query query object
  * @node1: inner algebra node
  * @seq: sequence of order condition #rasqal_expression
+ * @distinct: distinct flag
  *
- * INTERNAL - Create a new ORDERBY algebra node for a sequence of order conditions
+ * INTERNAL - Create a new ORDERBY algebra node for a sequence of order conditions (with optional DISTINCTness)
  * 
  * #node and #seq become owned by the new node
  *
@@ -278,7 +278,8 @@ rasqal_new_leftjoin_algebra_node(rasqal_query* query,
 rasqal_algebra_node*
 rasqal_new_orderby_algebra_node(rasqal_query* query,
                                 rasqal_algebra_node* node1,
-                                raptor_sequence* seq)
+                                raptor_sequence* seq,
+                                int distinct)
 {
   rasqal_algebra_node* node;
 
@@ -289,6 +290,7 @@ rasqal_new_orderby_algebra_node(rasqal_query* query,
   if(node) {
     node->node1 = node1;
     node->seq = seq;
+    node->distinct = distinct;
     
     return node;
   }
@@ -303,6 +305,47 @@ rasqal_new_orderby_algebra_node(rasqal_query* query,
 }
 
 
+/*
+ * rasqal_new_slice_algebra_node:
+ * @query: #rasqal_query query object
+ * @node1: inner algebra node
+ * @limit: max rows limit (or <0 for no limit)
+ * @offset: start row offset (or <0 for no offset)
+ *
+ * INTERNAL - Create a new SLICE algebra node for selecting a range of rows
+ * 
+ * #node and #seq become owned by the new node
+ *
+ * Return value: a new #rasqal_algebra_node object or NULL on failure
+ **/
+rasqal_algebra_node*
+rasqal_new_slice_algebra_node(rasqal_query* query,
+                              rasqal_algebra_node* node1,
+                              int limit,
+                              int offset)
+{
+  rasqal_algebra_node* node;
+
+  if(!query || !node1)
+    goto fail;
+
+  node = rasqal_new_algebra_node(query, RASQAL_ALGEBRA_OPERATOR_SLICE);
+  if(node) {
+    node->node1 = node1;
+    node->limit = limit;
+    node->offset = offset;
+    
+    return node;
+  }
+
+  fail:
+  if(node1)
+    rasqal_free_algebra_node(node1);
+
+  return NULL;
+}
+
+
 /*
  * rasqal_new_project_algebra_node:
  * @query: #rasqal_query query object
@@ -627,7 +670,7 @@ rasqal_free_algebra_node(rasqal_algebra_node* node)
  * Get the algebra node operator .
  * 
  * The operator for the given algebra node. See also
- * rasqal_algebra_node_operator_as_string().
+ * rasqal_algebra_node_operator_as_counted_string().
  *
  * Return value: algebra node operator
  **/
@@ -638,45 +681,52 @@ rasqal_algebra_node_get_operator(rasqal_algebra_node* node)
 }
 
 
-static const char* const
-rasqal_algebra_node_operator_labels[RASQAL_ALGEBRA_OPERATOR_LAST + 1] = {
-  "UNKNOWN",
-  "BGP",
-  "Filter",
-  "Join",
-  "Diff",
-  "LeftJoin",
-  "Union",
-  "ToList",
-  "OrderBy",
-  "Project",
-  "Distinct",
-  "Reduced",
-  "Slice",
-  "Graph",
-  "Assignment",
-  "Group",
-  "Aggregate",
-  "Having"
+static struct {
+  const char* const label;
+  size_t length;
+} rasqal_algebra_node_operator_labels[RASQAL_ALGEBRA_OPERATOR_LAST + 1] = {
+  { "UNKNOWN", 7 },
+  { "BGP" , 3 },
+  { "Filter", 6 },
+  { "Join", 4 },
+  { "Diff", 4 },
+  { "LeftJoin", 8 },
+  { "Union", 5 },
+  { "ToList", 6 },
+  { "OrderBy", 7 },
+  { "Project", 7 },
+  { "Distinct", 8 },
+  { "Reduced", 7 },
+  { "Slice", 5 },
+  { "Graph", 5 },
+  { "Assignment", 10 },
+  { "Group", 5 },
+  { "Aggregate", 9 },
+  { "Having", 6 }
 };
 
 
 /**
- * rasqal_algebra_node_operator_as_string:
+ * rasqal_algebra_node_operator_as_counted_string:
  * @op: the #rasqal_algebra_node_operator verb of the query
+ * @length_p: pointer to store the length (or NULL)
  *
- * Get a string for the query verb.
+ * INTERNAL - Get a counted string for the query verb.
  * 
  * Return value: pointer to a shared string label for the query verb
  **/
 const char*
-rasqal_algebra_node_operator_as_string(rasqal_algebra_node_operator op)
+rasqal_algebra_node_operator_as_counted_string(rasqal_algebra_node_operator op,
+                                               size_t* length_p)
 {
   if(op <= RASQAL_ALGEBRA_OPERATOR_UNKNOWN || 
      op > RASQAL_ALGEBRA_OPERATOR_LAST)
     op = RASQAL_ALGEBRA_OPERATOR_UNKNOWN;
 
-  return rasqal_algebra_node_operator_labels[(int)op];
+  if(length_p)
+    *length_p = rasqal_algebra_node_operator_labels[(int)op].length;
+
+  return rasqal_algebra_node_operator_labels[(int)op].label;
 }
   
 
@@ -696,22 +746,26 @@ rasqal_algebra_write_indent(raptor_iostream *iostr, int indent)
 
 static int
 rasqal_algebra_algebra_node_write_internal(rasqal_algebra_node *node, 
-                                           raptor_iostream* iostr, int indent)
+                                           raptor_iostream* iostr,
+                                           unsigned int indent)
 {
-  const char* op_string = rasqal_algebra_node_operator_as_string(node->op);
+  const char* op_string;
+  size_t op_length;
   int arg_count = 0;
-  int indent_delta;
+  unsigned int indent_delta;
+
+  op_string = rasqal_algebra_node_operator_as_counted_string(node->op,
+                                                             &op_length);
   
   if(node->op == RASQAL_ALGEBRA_OPERATOR_BGP && !node->triples) {
     raptor_iostream_write_byte('Z', iostr);
     return 0;
   }
-  
-  indent_delta = strlen(op_string);
 
-  raptor_iostream_counted_string_write(op_string, indent_delta, iostr);
+  raptor_iostream_counted_string_write(op_string, op_length, iostr);
   raptor_iostream_counted_string_write("(\n", 2, iostr);
-  indent_delta++;
+
+  indent_delta = (unsigned int)op_length + 1;
   
   indent += indent_delta;
   rasqal_algebra_write_indent(iostr, indent);
@@ -814,10 +868,10 @@ rasqal_algebra_algebra_node_write_internal(rasqal_algebra_node *node,
       raptor_iostream_counted_string_write(" ,\n", 3, iostr);
       rasqal_algebra_write_indent(iostr, indent);
     }
-    raptor_iostream_string_write("slice start ", iostr);
-    raptor_iostream_decimal_write(node->start, iostr);
-    raptor_iostream_string_write(" length ", iostr);
-    raptor_iostream_decimal_write(node->length, iostr);
+    raptor_iostream_string_write("slice limit ", iostr);
+    raptor_iostream_decimal_write(node->limit, iostr);
+    raptor_iostream_string_write(" offset ", iostr);
+    raptor_iostream_decimal_write(node->offset, iostr);
     raptor_iostream_write_byte('\n', iostr);
     arg_count++;
   }
@@ -1251,9 +1305,59 @@ static rasqal_algebra_node*
 rasqal_algebra_select_graph_pattern_to_algebra(rasqal_query* query,
                                                rasqal_graph_pattern* gp)
 {
-  rasqal_log_error_simple(query->world, RAPTOR_LOG_LEVEL_ERROR,
-                          NULL, "Sub SELECT execution is not implemented");
+  rasqal_projection* projection;
+  rasqal_solution_modifier* modifier;
+  rasqal_graph_pattern* where_gp;
+  rasqal_algebra_node* where_node;
+  rasqal_algebra_node* node;
+  rasqal_algebra_aggregate* ae;
+  
+  where_gp = rasqal_graph_pattern_get_sub_graph_pattern(gp, 0);
+  projection = gp->projection;
+  modifier = gp->modifier;
 
+  where_node = rasqal_algebra_graph_pattern_to_algebra(query, where_gp);
+  if(!where_node)
+    goto fail;
+
+  node = rasqal_algebra_query_add_group_by(query, where_node, modifier);
+  where_node = NULL; /* now owned by node */
+  if(!node)
+    goto fail;
+
+  ae = rasqal_algebra_query_prepare_aggregates(query, node, projection,
+                                               modifier);
+  if(!ae)
+    goto fail;
+
+  if(ae) {
+    node = rasqal_algebra_query_add_aggregation(query, ae, node);
+    ae = NULL;
+    if(!node)
+      goto fail;
+  }
+
+  node = rasqal_algebra_query_add_having(query, node, modifier);
+  if(!node)
+    goto fail;
+
+  node = rasqal_algebra_query_add_projection(query, node, projection);
+  if(!node)
+    goto fail;
+
+  node = rasqal_algebra_query_add_orderby(query, node, projection, modifier);
+  if(!node)
+    goto fail;
+
+  node = rasqal_algebra_query_add_distinct(query, node, projection);
+  if(!node)
+    goto fail;
+
+  node = rasqal_algebra_query_add_slice(query, node, modifier);
+
+  return node;
+
+  fail:
   return NULL;
 }
 
@@ -1392,10 +1496,10 @@ rasqal_algebra_remove_znodes(rasqal_query* query, rasqal_algebra_node* node,
   rasqal_algebra_node_print(node, stderr);
   fprintf(stderr, "\nNode 1 (%s): %s\n", 
           is_z1 ? "empty" : "not empty",
-          rasqal_algebra_node_operator_as_string(node->node1->op));
+          rasqal_algebra_node_operator_as_counted_string(node->node1->op, NULL));
   fprintf(stderr, "Node 2 (%s): %s\n", 
           is_z2 ? "empty" : "not empty",
-          rasqal_algebra_node_operator_as_string(node->node2->op));
+          rasqal_algebra_node_operator_as_counted_string(node->node2->op, NULL));
 #endif
 
   if(is_z1 && !is_z2) {
@@ -1507,7 +1611,7 @@ rasqal_algebra_extract_aggregate_expression_visit(void *user_data,
 
 
   /* is expression is in map? */
-  v = rasqal_map_search(ae->agg_vars, e);
+  v = (rasqal_variable*)rasqal_map_search(ae->agg_vars, e);
   if(v) {
     /* Yes: get value => use existing internal variable for it */
     RASQAL_DEBUG2("Found variable %s for existing expression\n", v->name);
@@ -1541,7 +1645,7 @@ rasqal_algebra_extract_aggregate_expression_visit(void *user_data,
     /* If not an error, create a new internal variable name for it
      * $$agg{id}$$ and add it to the map.
      */
-    var_name = (char*)RASQAL_MALLOC(cstring, 20);
+    var_name = RASQAL_MALLOC(char*, 20);
     if(!var_name) {
       ae->error = 1;
       return 1;
@@ -1606,14 +1710,15 @@ rasqal_algebra_extract_aggregate_expression_visit(void *user_data,
 static int
 rasqal_algebra_extract_aggregate_expressions(rasqal_query* query,
                                              rasqal_algebra_node* node,
-                                             rasqal_algebra_aggregate* ae)
+                                             rasqal_algebra_aggregate* ae,
+                                             rasqal_projection* projection)
 {
   raptor_sequence* seq;
   int i;
   int rc = 0;
   rasqal_variable* v;
   
-  if(query->verb != RASQAL_QUERY_VERB_SELECT)
+  if(!projection)
     return 0;
 
   ae->query = query;
@@ -1649,19 +1754,21 @@ rasqal_algebra_extract_aggregate_expressions(rasqal_query* query,
    * expressions into the ae->agg_vars map, replacing them with
    * internal variable names.
    */
-  for(seq = query->selects, i = 0;
-      (v = (rasqal_variable*)raptor_sequence_get_at(seq, i));
-      i++) {
-    rasqal_expression* expr = v->expression;
+  if ((seq = projection->variables)) {
+    for(i = 0;
+	(v = (rasqal_variable*)raptor_sequence_get_at(seq, i));
+	i++) {
+      rasqal_expression* expr = v->expression;
 
-    if(!expr)
-      continue;
+      if(!expr)
+	continue;
 
-    if(rasqal_expression_visit(expr,
-                               rasqal_algebra_extract_aggregate_expression_visit,
-                               ae)) {
-      rc = 1;
-      goto tidy;
+      if(rasqal_expression_visit(expr,
+				 rasqal_algebra_extract_aggregate_expression_visit,
+				 ae)) {
+	rc = 1;
+	goto tidy;
+      }
     }
   }
 
@@ -1765,6 +1872,8 @@ rasqal_algebra_replace_aggregate_expressions(rasqal_query* query,
  * rasqal_algebra_query_prepare_aggregates:
  * @query: #rasqal_query to read from
  * @node: algebra node to prepare
+ * @projection: variable projection to use
+ * @modifier: solution modifier to use
  *
  * INTERNAL - prepare query aggregates
  *
@@ -1772,16 +1881,17 @@ rasqal_algebra_replace_aggregate_expressions(rasqal_query* query,
  */
 rasqal_algebra_aggregate*
 rasqal_algebra_query_prepare_aggregates(rasqal_query* query,
-                                        rasqal_algebra_node* node)
+                                        rasqal_algebra_node* node,
+                                        rasqal_projection* projection,
+                                        rasqal_solution_modifier* modifier)
 {
   rasqal_algebra_aggregate* ae;
   
-  ae = (rasqal_algebra_aggregate*)RASQAL_CALLOC(rasqal_algebra_aggregate,
-                                                sizeof(*ae), 1);
+  ae = RASQAL_CALLOC(rasqal_algebra_aggregate*, 1, sizeof(*ae));
   if(!ae)
     return NULL;
 
-  if(rasqal_algebra_extract_aggregate_expressions(query, node, ae)) {
+  if(rasqal_algebra_extract_aggregate_expressions(query, node, ae, projection)) {
     RASQAL_DEBUG1("rasqal_algebra_extract_aggregate_expressions() failed\n");
     rasqal_free_algebra_aggregate(ae);
     rasqal_free_algebra_node(node);
@@ -1790,13 +1900,15 @@ rasqal_algebra_query_prepare_aggregates(rasqal_query* query,
 
   /* Update variable use structures since agg variables were created */
   if(ae->counter)
-    rasqal_query_build_variables_use(query);
+    rasqal_query_build_variables_use(query, projection);
   
 #if RASQAL_DEBUG
   if(ae->counter) {
-    if(query->selects) {
+    raptor_sequence* seq = projection->variables;
+
+    if(seq) {
       RASQAL_DEBUG1("after aggregate expressions extracted:\n");
-      raptor_sequence_print(query->selects, stderr);
+      raptor_sequence_print(seq, stderr);
       fputs("\n", stderr);
 
       RASQAL_DEBUG1("aggregate expressions:\n");
@@ -1812,9 +1924,9 @@ rasqal_algebra_query_prepare_aggregates(rasqal_query* query,
   /* if agg expressions were found, need to walk HAVING list and do a
    * similar replacement substituion in the expressions
    */
-  if(ae->counter && query->modifier && query->modifier->having_conditions) {
+  if(ae->counter && modifier && modifier->having_conditions) {
     if(rasqal_algebra_replace_aggregate_expressions(query, 
-                                                    query->modifier->having_conditions,
+                                                    modifier->having_conditions,
                                                     ae)) {
       RASQAL_DEBUG1("rasqal_algebra_replace_aggregate_expressions() failed\n");
       rasqal_free_algebra_aggregate(ae);
@@ -1831,6 +1943,7 @@ rasqal_algebra_query_prepare_aggregates(rasqal_query* query,
  * rasqal_algebra_query_add_group_by:
  * @query: #rasqal_query to read from
  * @node: node to apply modifiers to
+ * @modifier: solution modifier to use
  *
  * Apply any needed GROUP BY to query algebra structure
  *
@@ -1838,15 +1951,16 @@ rasqal_algebra_query_prepare_aggregates(rasqal_query* query,
  */
 rasqal_algebra_node*
 rasqal_algebra_query_add_group_by(rasqal_query* query,
-                                  rasqal_algebra_node* node)
+                                  rasqal_algebra_node* node,
+                                  rasqal_solution_modifier* modifier)
 {
   raptor_sequence* modifier_seq;
   
-  if(!query->modifier)
+  if(!modifier)
     return node;
   
   /* GROUP BY */
-  modifier_seq = query->modifier->group_conditions;
+  modifier_seq = modifier->group_conditions;
 
   if(modifier_seq) {
     raptor_sequence* seq;
@@ -1874,25 +1988,30 @@ rasqal_algebra_query_add_group_by(rasqal_query* query,
   
 
 /**
- * rasqal_algebra_query_add_modifiers:
+ * rasqal_algebra_query_add_orderby:
  * @query: #rasqal_query to read from
  * @node: node to apply modifiers to
+ * @projection: variable projection to use
+ * @modifier: solution modifier to use
  *
  * Apply any needed modifiers to query algebra structure
  *
  * Return value: non-0 on failure
  */
 rasqal_algebra_node*
-rasqal_algebra_query_add_modifiers(rasqal_query* query,
-                                   rasqal_algebra_node* node)
+rasqal_algebra_query_add_orderby(rasqal_query* query,
+                                   rasqal_algebra_node* node,
+                                   rasqal_projection* projection,
+                                   rasqal_solution_modifier* modifier)
 {
   raptor_sequence* modifier_seq;
-  
-  if(!query->modifier)
+  int distinct = 0;
+
+  if(!modifier)
     return node;
   
   /* ORDER BY */
-  modifier_seq = query->modifier->order_conditions;
+  modifier_seq = modifier->order_conditions;
   if(modifier_seq) {
     raptor_sequence* seq;
     
@@ -1904,8 +2023,11 @@ rasqal_algebra_query_add_modifiers(rasqal_query* query,
       rasqal_free_algebra_node(node);
       return NULL;
     }
-    
-    node = rasqal_new_orderby_algebra_node(query, node, seq);
+
+    if(projection)
+      distinct = projection->distinct;
+
+    node = rasqal_new_orderby_algebra_node(query, node, seq, distinct);
     
 #if RASQAL_DEBUG > 1
     RASQAL_DEBUG1("modified after adding orderby node, algebra node now:\n  ");
@@ -1918,6 +2040,42 @@ rasqal_algebra_query_add_modifiers(rasqal_query* query,
 }
 
 
+/**
+ * rasqal_algebra_query_add_slice:
+ * @query: #rasqal_query to read from
+ * @node: node to apply modifiers to
+ * @modifier: solution modifier to use
+ *
+ * Apply any needed slice (LIMIT, OFFSET) modifiers to query algebra structure
+ *
+ * This is separate from rasqal_algebra_query_add_orderby() since currently
+ * the query results module implements that for the outer result rows.
+ *
+ * Return value: non-0 on failure
+ */
+rasqal_algebra_node*
+rasqal_algebra_query_add_slice(rasqal_query* query,
+                               rasqal_algebra_node* node,
+                               rasqal_solution_modifier* modifier)
+{
+  if(!modifier)
+    return node;
+  
+  /* LIMIT and OFFSET */
+  if(modifier->limit >= 0 || modifier->offset > 0) {
+    node = rasqal_new_slice_algebra_node(query, node, modifier->limit, modifier->offset);
+
+#if RASQAL_DEBUG > 1
+    RASQAL_DEBUG1("modified after adding slice node, algebra node now:\n  ");
+    rasqal_algebra_node_print(node, stderr);
+    fputs("\n", stderr);
+#endif
+  }
+
+  return node;
+}
+
+
 /**
  * rasqal_algebra_query_add_aggregation:
  * @query: #rasqal_query to read from
@@ -1982,36 +2140,32 @@ rasqal_algebra_query_add_aggregation(rasqal_query* query,
  * rasqal_algebra_query_add_projection:
  * @query: #rasqal_query to read from
  * @node: node to apply projection to
+ * @projection: variable projection to use
  *
- * Apply query projection to query algebra structure
+ * Add a projection to the query algebra structure
  *
  * Return value: non-0 on failure
  */
 rasqal_algebra_node*
 rasqal_algebra_query_add_projection(rasqal_query* query,
-                                   rasqal_algebra_node* node)
+                                    rasqal_algebra_node* node,
+                                    rasqal_projection* projection)
 {
   int vars_size = 0;
   raptor_sequence* seq = NULL;  /* sequence of rasqal_variable* */
   raptor_sequence* vars_seq;
   int i;
+
+  if(!projection)
+    return NULL;
   
-  /* FIXME - do not always need a PROJECT node when the variables at
-   * the top level node are the same as the projection list.
+  /* FIXME Optimization: do not always need a PROJECT node when the
+   * variables at the top level node are the same as the projection
+   * list.
    */
-  if(query->verb == RASQAL_QUERY_VERB_SELECT)
-    /* project all selected variables */
-    seq = query->selects;
-  else if(query->verb == RASQAL_QUERY_VERB_CONSTRUCT) {
-    /* project all variables mentioned in CONSTRUCT  */
-    seq = rasqal_algebra_get_variables_mentioned_in(query, 
-                                                    RASQAL_VAR_USE_MAP_OFFSET_VERBS);
-    if(!seq) {
-      rasqal_free_algebra_node(node);
-      return NULL;
-    }
-  }
-  
+
+  /* project all projection variables (may be an empty sequence) */
+  seq = projection->variables;
   if(seq)
     vars_size = raptor_sequence_size(seq);
   
@@ -2038,8 +2192,62 @@ rasqal_algebra_query_add_projection(rasqal_query* query,
   fputs("\n", stderr);
 #endif
   
-  if(query->verb == RASQAL_QUERY_VERB_CONSTRUCT && seq) 
-    raptor_free_sequence(seq);
+  return node;
+}
+
+
+/**
+ * rasqal_algebra_query_add_construct_projection:
+ * @query: #rasqal_query to read from
+ * @node: node to apply projection to
+ *
+ * Add a query projection for a CONSTRUCT to the query algebra structure
+ *
+ * Return value: non-0 on failure
+ */
+rasqal_algebra_node*
+rasqal_algebra_query_add_construct_projection(rasqal_query* query,
+                                              rasqal_algebra_node* node)
+{
+  int vars_size = 0;
+  raptor_sequence* seq = NULL;  /* sequence of rasqal_variable* */
+  raptor_sequence* vars_seq;
+  int i;
+  
+  /* project all variables mentioned in CONSTRUCT */
+  seq = rasqal_algebra_get_variables_mentioned_in(query, 
+                                                  RASQAL_VAR_USE_MAP_OFFSET_VERBS);
+  if(!seq) {
+    rasqal_free_algebra_node(node);
+    return NULL;
+  }
+  
+  vars_size = raptor_sequence_size(seq);
+  
+  vars_seq = raptor_new_sequence((raptor_data_free_handler)rasqal_free_variable,
+                                 (raptor_data_print_handler)rasqal_variable_print);
+  if(!vars_seq) {
+    rasqal_free_algebra_node(node);
+    return NULL;
+  }
+  
+  for(i = 0; i < vars_size; i++) {
+    rasqal_variable* v;
+
+    v = (rasqal_variable*)raptor_sequence_get_at(seq, i);
+    v = rasqal_new_variable_from_variable(v);
+    raptor_sequence_push(vars_seq, v);
+  }
+  
+  node = rasqal_new_project_algebra_node(query, node, vars_seq);
+  
+#if RASQAL_DEBUG > 1
+  RASQAL_DEBUG1("modified after adding construct project node, algebra node now:\n  ");
+  rasqal_algebra_node_print(node, stderr);
+  fputs("\n", stderr);
+#endif
+  
+  raptor_free_sequence(seq);
 
   return node;
 }
@@ -2049,6 +2257,7 @@ rasqal_algebra_query_add_projection(rasqal_query* query,
  * rasqal_algebra_query_add_distinct:
  * @query: #rasqal_query to read from
  * @node: node to apply distinct to
+ * @projection: variable projection to use
  *
  * Apply distinctness to query algebra structure
  *
@@ -2056,9 +2265,13 @@ rasqal_algebra_query_add_projection(rasqal_query* query,
  */
 rasqal_algebra_node*
 rasqal_algebra_query_add_distinct(rasqal_query* query,
-                                  rasqal_algebra_node* node)
+                                  rasqal_algebra_node* node,
+                                  rasqal_projection* projection)
 {
-  if(query->distinct) {
+  if(!projection)
+    return node;
+
+  if(projection->distinct) {
     node = rasqal_new_distinct_algebra_node(query, node);
 
 #if RASQAL_DEBUG > 1
@@ -2076,6 +2289,7 @@ rasqal_algebra_query_add_distinct(rasqal_query* query,
  * rasqal_algebra_query_add_having:
  * @query: #rasqal_query to read from
  * @node: node to apply having to
+ * @modifier: solution modifier to use
  *
  * Apply any needed HAVING expressions to query algebra structure
  *
@@ -2083,15 +2297,16 @@ rasqal_algebra_query_add_distinct(rasqal_query* query,
  */
 rasqal_algebra_node*
 rasqal_algebra_query_add_having(rasqal_query* query,
-                                rasqal_algebra_node* node)
+                                rasqal_algebra_node* node,
+                                rasqal_solution_modifier* modifier)
 {
   raptor_sequence* modifier_seq;
   
-  if(!query->modifier)
+  if(!modifier)
     return node;
   
   /* HAVING */
-  modifier_seq = query->modifier->having_conditions;
+  modifier_seq = modifier->having_conditions;
   if(modifier_seq) {
     raptor_sequence* exprs_seq;
     
@@ -2372,7 +2587,7 @@ main(int argc, char *argv[]) {
   raptor_sequence_push(conditions, expr4);
   expr4 = NULL; /* now owned by conditions */
   
-  node9 = rasqal_new_orderby_algebra_node(query, node7, conditions);
+  node9 = rasqal_new_orderby_algebra_node(query, node7, conditions, 0);
   if(!node9)
     FAIL;
   /* these become owned by node9 */
