@@ -34,8 +34,15 @@
 #include <stdio.h>
 #include <string.h>
 #include <ctype.h>
-#ifdef HAVE_SYS_TIME_H
-#include <sys/time.h>
+#ifdef TIME_WITH_SYS_TIME
+# include <sys/time.h>
+# include <time.h>
+#else
+# ifdef HAVE_SYS_TIME_H
+#  include <sys/time.h>
+# else
+#  include <time.h>
+# endif
 #endif
 #ifdef HAVE_STDLIB_H
 #include <stdlib.h>
@@ -205,7 +212,8 @@ rasqal_xsd_datetime_parse(const char *datetime_string,
                           int is_dateTime)
 {
   const char *p, *q; 
-  char b[16];
+#define B_SIZE 16
+  char b[B_SIZE];
   unsigned int l, t, t2, is_neg;
   unsigned long u;
 #define MICROSECONDS_MAX_DIGITS 6
@@ -225,7 +233,7 @@ rasqal_xsd_datetime_parse(const char *datetime_string,
   }
   for(q = p; ISNUM(*p); p++)
     ;
-  l = p - q;
+  l = (unsigned int)(p - q);
   
   /* error if
      - less than 4 digits in year
@@ -235,7 +243,9 @@ rasqal_xsd_datetime_parse(const char *datetime_string,
   if(l < 4 || (l > 4 && *q=='0') || *p != '-')
     return -1;
 
-  l = (l < sizeof(b)-1 ? l : sizeof(b)-1);
+  if(l >= (B_SIZE - 1))
+     l = (unsigned int)(B_SIZE - 1);
+
   memcpy(b, q, l);
   b[l] = 0; /* ensure nul termination */
   u = strtoul(b, 0, 10);
@@ -255,7 +265,7 @@ rasqal_xsd_datetime_parse(const char *datetime_string,
   
   for(q = ++p; ISNUM(*p); p++)
     ;
-  l = p - q;
+  l = (unsigned int)(p - q);
   
   /* error if month is not 2 digits or '-' is not the separator */
   if(l != 2 || *p != '-')
@@ -274,7 +284,7 @@ rasqal_xsd_datetime_parse(const char *datetime_string,
   
   for(q = ++p; ISNUM(*p); p++)
     ;
-  l = p - q;
+  l = (unsigned int)(p - q);
 
   if(is_dateTime) {
     /* xsd:dateTime: error if day is not 2 digits or 'T' is not the separator */
@@ -304,7 +314,7 @@ rasqal_xsd_datetime_parse(const char *datetime_string,
 
     for(q = ++p; ISNUM(*p); p++)
       ;
-    l = p - q;
+    l = (unsigned int)(p - q);
 
     /* error if hour is not 2 digits or ':' is not the separator */
     if(l != 2 || *p != ':')
@@ -325,7 +335,7 @@ rasqal_xsd_datetime_parse(const char *datetime_string,
 
     for(q = ++p; ISNUM(*p); p++)
       ;
-    l = p - q;
+    l = (unsigned int)(p - q);
 
     /* error if minute is not 2 digits or ':' is not the separator */
     if(l != 2 || *p != ':')
@@ -346,7 +356,7 @@ rasqal_xsd_datetime_parse(const char *datetime_string,
 
     for(q = ++p; ISNUM(*p); p++)
       ;
-    l = p - q;
+    l = (unsigned int)(p - q);
 
     /* error if second is not 2 digits or separator is not 
      * '.' (second fraction)
@@ -387,7 +397,7 @@ rasqal_xsd_datetime_parse(const char *datetime_string,
 
       if(!(*q == '0' && q == p)) {
         /* allow ".0" */
-        l = p - q;
+        l = (unsigned int)(p - q);
 
         if(l < 1) /* need at least 1 num */
           return -8;
@@ -437,7 +447,7 @@ rasqal_xsd_datetime_parse(const char *datetime_string,
       /* timezone hours */
       for(q = ++p; ISNUM(*p); p++)
         ;
-      l = p - q;
+      l = (unsigned int)(p - q);
       if(l != 2 || *p!=':')
         return -9;
 
@@ -455,7 +465,7 @@ rasqal_xsd_datetime_parse(const char *datetime_string,
       /* timezone minutes */    
       for(q = ++p; ISNUM(*p); p++)
         ;
-      l = p - q;
+      l = (unsigned int)(p - q);
       if(l != 2)
         return -10;
 
@@ -529,7 +539,7 @@ rasqal_new_xsd_datetime(rasqal_world* world, const char *datetime_string)
   rasqal_xsd_datetime* dt;
   int rc = 0;
   
-  dt = (rasqal_xsd_datetime*)RASQAL_MALLOC(datetime, sizeof(*dt));
+  dt = RASQAL_MALLOC(rasqal_xsd_datetime*, sizeof(*dt));
   if(!dt)
     return NULL;
   
@@ -560,7 +570,7 @@ rasqal_new_xsd_datetime_from_unixtime(rasqal_world* world, time_t secs)
   rasqal_xsd_datetime* dt;
   int rc = 0;
   
-  dt = (rasqal_xsd_datetime*)RASQAL_MALLOC(datetime, sizeof(*dt));
+  dt = RASQAL_MALLOC(rasqal_xsd_datetime*, sizeof(*dt));
   if(!dt)
     return NULL;
 
@@ -589,7 +599,7 @@ rasqal_new_xsd_datetime_from_timeval(rasqal_world* world, struct timeval *tv)
   rasqal_xsd_datetime* dt;
   int rc = 0;
   
-  dt = (rasqal_xsd_datetime*)RASQAL_MALLOC(datetime, sizeof(*dt));
+  dt = RASQAL_MALLOC(rasqal_xsd_datetime*, sizeof(*dt));
   if(!dt)
     return NULL;
 
@@ -619,40 +629,104 @@ rasqal_free_xsd_datetime(rasqal_xsd_datetime* dt)
 }
 
 
-#define TIMEZONE_BUFFER_LEN 7
+#define TIMEZONE_BUFFER_LEN 6
 
 /*
  * rasqal_xsd_datetime_timezone_format:
  * @dt: datetime
  * @buffer: buffer to write the formatted timezone
- * @len: length of buffer; must be 7 or larger
+ * @bufsize: length of @buffer; must be 7 or larger
  *
  * INTERNAL - format a timezone into the passed in buffer
  *
- * Reutnr value: non-0 on failure
+ * Return value: size of buffer or 0 on failure
  */
 static int
 rasqal_xsd_datetime_timezone_format(const rasqal_xsd_datetime *dt,
-                                    char* buffer, size_t len)
+                                    char* buffer, size_t bufsize)
 {
   int mins;
+  size_t tz_len;
   
-  if(!buffer || len < TIMEZONE_BUFFER_LEN)
-    return 1;
+  if(!buffer || !bufsize)
+    return -1;
   
   mins = abs(dt->timezone_minutes);
-  if(mins == RASQAL_XSD_DATETIME_NO_TZ)
+  if(mins == RASQAL_XSD_DATETIME_NO_TZ) {
+    tz_len = 0;
+
+    if(bufsize < (tz_len + 1))
+      return -1;
+
     buffer[0] = '\0';
-  else if(!mins)
-    memcpy(buffer, "Z", 2);
-  else {
-    int hrs = (mins / 60);
-    snprintf(buffer, len, "%c%02d:%02d", 
-             (mins != dt->timezone_minutes ? '-' : '+'),
-             hrs, mins);
+  } else if(!mins) {
+    tz_len = 1;
+    if(bufsize < (tz_len + 1))
+      return -1;
+
+    buffer[0] = 'Z';
+    buffer[1] = '\0';
+  } else {
+    int hours;
+
+    tz_len = TIMEZONE_BUFFER_LEN;
+
+    if(bufsize < (tz_len + 1))
+      return -1;
+
+    buffer[0] = (mins != dt->timezone_minutes ? '-' : '+');
+
+    hours = (mins / 60);
+    buffer[1] = (hours / 10) + '0';
+    hours -= hours * 10;
+    buffer[2] = hours + '0';
+    buffer[3] = ':';
+
+    mins -= hours * 60;
+    buffer[4] = (mins / 10) + '0';
+    mins -= mins * 10;
+    buffer[5] = mins + '0';
+
+    buffer[6] = '\0';
   }
 
-  return 0;
+  return (int)tz_len;
+}
+
+
+static int
+rasqal_xsd_format_microseconds(char* buffer, size_t bufsize, 
+                               unsigned int microseconds)
+{
+  int len = 0;
+  char *p;
+  unsigned int value;
+  unsigned int base = 10;
+  unsigned int multiplier;
+
+  value = microseconds;
+  multiplier = 100000;
+  do {
+    value = value % multiplier;
+    multiplier /= base;
+    len++;
+  } while(value && multiplier);
+
+  if(!buffer || (int)bufsize < (len + 1)) /* +1 for NUL */
+    return len;
+
+  value = microseconds;
+  multiplier = 100000;
+  p = buffer;
+  do {
+    unsigned digit = value / multiplier;
+    *p++ = '0' + digit;
+    value = value % multiplier;
+    multiplier /= base;
+  } while(value && multiplier);
+  *p = '\0';
+
+  return len;
 }
 
 
@@ -671,74 +745,66 @@ char*
 rasqal_xsd_datetime_to_counted_string(const rasqal_xsd_datetime *dt,
                                       size_t *len_p)
 {
-  char *ret = 0;
-  int is_neg;
-  int r = 0;
-  int i;
+  size_t len;
+  char *buffer = NULL;
+  char *p;
   /* "[+-]HH:MM\0" */
-  char timezone_string[TIMEZONE_BUFFER_LEN];
+  char timezone_string[TIMEZONE_BUFFER_LEN + 1];
+  int year_len;
+  int tz_string_len;
+  size_t microseconds_len = 0;
   
   if(!dt)
     return NULL;
     
-  is_neg = dt->year < 0;
-
-  if(rasqal_xsd_datetime_timezone_format(dt, timezone_string,
-                                         TIMEZONE_BUFFER_LEN))
+  tz_string_len = rasqal_xsd_datetime_timezone_format(dt, timezone_string,
+                                                      TIMEZONE_BUFFER_LEN + 1);
+  if(tz_string_len < 0)
     return NULL;
 
-  /* format twice: first with null buffer of zero size to get the
-   * required buffer size second time to the allocated buffer
-   */
-  for(i = 0; i < 2; i++) { 
-    if(dt->microseconds) {
-      char microsecs[9];
-      int j;
-      
-      snprintf(microsecs, 9, "%.6f", ((double)dt->microseconds) / 1000000);
-      for(j = strlen(microsecs) - 1; j > 2 && microsecs[j] == '0'; j--)
-        microsecs[j] = '\0';
-      
-      r = snprintf(ret, r, "%s%04d-%2.2d-%2.2dT%2.2d:%2.2d:%2.2d%s%s",
-                   is_neg ? "-" : "",
-                   is_neg ? -dt->year : dt->year,
-                   dt->month,
-                   dt->day,
-                   dt->hour,
-                   dt->minute,
-                   dt->second,
-                   microsecs+1,
-                   timezone_string);
-    } else {
-      r = snprintf((char*)ret, r, "%s%04d-%2.2d-%2.2dT%2.2d:%2.2d:%2.2d%s",
-                   is_neg ? "-" : "",
-                   is_neg ? -dt->year : dt->year,
-                   dt->month,
-                   dt->day,
-                   dt->hour,
-                   dt->minute,
-                   dt->second,
-                   timezone_string);
-    }
-    
-    /* error? */
-    if(r < 0) {
-      if(ret)
-        RASQAL_FREE(cstring, ret);
-      return NULL;
-    }
-
-    /* alloc return buffer on first pass */
-    if(!i) {
-      if(len_p)
-        *len_p = r;
-
-      ret = (char*)RASQAL_MALLOC(cstring, ++r);
-      if(!ret)
-        return NULL;
-    }
+  year_len = rasqal_format_integer(NULL, 0, dt->year, 4, '0');
+  
+  len = year_len +
+        15 + /* "-MM-DDTHH:MM:SS" = 15 */
+        tz_string_len;
+  if(dt->microseconds) {
+    microseconds_len = rasqal_xsd_format_microseconds(NULL, 0, dt->microseconds);
+    len += 1 /* . */ + microseconds_len;
   }
-  return ret;
+  
+  if(len_p)
+    *len_p = len;
+    
+  buffer = RASQAL_MALLOC(char*, len + 1);
+  if(!buffer)
+    return NULL;
+
+  p = buffer;
+  p += rasqal_format_integer(p, year_len + 1, dt->year, 4, '0');
+  *p++ = '-';
+  p += rasqal_format_integer(p, 2 + 1, dt->month, 2, '0');
+  *p++ = '-';
+  p += rasqal_format_integer(p, 2 + 1, dt->day, 2, '0');
+  *p++ = 'T';
+
+  p += rasqal_format_integer(p, 2 + 1, dt->hour, 2, '0');
+  *p++ = ':';
+  p += rasqal_format_integer(p, 2 + 1, dt->minute, 2, '0');
+  *p++ = ':';
+  p += rasqal_format_integer(p, 2 + 1, dt->second, 2, '0');
+
+  if(dt->microseconds) {
+    *p++ = '.';
+    p += rasqal_xsd_format_microseconds(p, microseconds_len + 1, dt->microseconds);
+  }
+  if(tz_string_len) {
+    memcpy(p, timezone_string, tz_string_len);
+    p += tz_string_len;
+  }
+
+  *p = '\0';
+
+  return buffer;
 }
 
 
@@ -914,6 +980,9 @@ rasqal_xsd_datetime_get_seconds_as_decimal(rasqal_world* world,
 }
 
 
+/* xsd:date formatted length excluding formatted year length */
+#define DATE_BUFFER_LEN_NO_YEAR 7
+
 /**
  * rasqal_xsd_date_to_string:
  * @d: date struct
@@ -925,43 +994,50 @@ rasqal_xsd_datetime_get_seconds_as_decimal(rasqal_world* world,
  * Return value: lexical form string or NULL on failure.
  */
 static char*
-rasqal_xsd_date_to_string(const rasqal_xsd_date *d)
+rasqal_xsd_date_to_string(const rasqal_xsd_date *date)
 {
-  char *ret = 0;
-  int is_neg;
-  int r = 0;
-  int i;
-   
-  if(!d)
+  char *buffer = NULL;
+  size_t len = DATE_BUFFER_LEN_NO_YEAR;
+  char *p;
+  int value;
+  unsigned int d;
+  int year_len;
+  
+  if(!date)
     return NULL;
     
-  is_neg = d->year < 0;
+  year_len = rasqal_format_integer(NULL, 0, date->year, -1, '\0');
 
-  /* format twice: first with null buffer of zero size to get the
-   * required buffer size second time to the allocated buffer
-   */
-  for(i = 0; i < 2; i++) {
-    r = snprintf((char*)ret, r, "%s%04d-%2.2d-%2.2d",
-                 is_neg ? "-" : "",
-                 is_neg ? -d->year : d->year,
-                 d->month,
-                 d->day);
+  buffer = RASQAL_MALLOC(char*, year_len + len + 1);
+  if(!buffer)
+    return NULL;
 
-    /* error? */
-    if(r < 0) {
-      if(ret)
-        RASQAL_FREE(cstring, ret);
-      return NULL;
-    }
+  p = buffer;
 
-    /* alloc return buffer on first pass */
-    if(!i) {
-      ret = (char*)RASQAL_MALLOC(cstring, ++r);
-      if(!ret)
-        return NULL;
-    }
-  }
-  return ret;
+  /* value is year; length can vary */
+  p += rasqal_format_integer(p, year_len + 1, date->year, -1, '\0');
+
+  *p++ = '-';
+
+  /* value is 2-digit month */
+  value = date->month;
+  d = (value / 10);
+  *p++ = d + '0';
+  value -= d * 10;
+  *p++ = value + '0';
+
+  *p++ = '-';
+
+  /* value is 2-digit day */
+  value = date->day;
+  d = (value / 10);
+  *p++ = d + '0';
+  value -= d * 10;
+  *p++ = value + '0';
+
+  *p = '\0';
+
+  return buffer;
 }
 
 
@@ -1184,7 +1260,7 @@ rasqal_xsd_datetime_get_as_unixtime(rasqal_xsd_datetime* dt)
     time_buf.tm_gmtoff = dt->timezone_minutes * 60;
 #endif
 
-  return timegm(&time_buf);
+  return rasqal_timegm(&time_buf);
 }
 
 
@@ -1207,7 +1283,7 @@ rasqal_xsd_datetime_get_as_timeval(rasqal_xsd_datetime *dt)
   if(!dt)
     return NULL;
   
-  tv = RASQAL_CALLOC(timeval, 1, sizeof(*tv));
+  tv = RASQAL_CALLOC(struct timeval*, 1, sizeof(*tv));
   if(!tv)
     return NULL;
   
@@ -1252,7 +1328,7 @@ rasqal_xsd_datetime_get_timezone_as_counted_string(rasqal_xsd_datetime *dt,
   if(!dt)
     return NULL;
   
-  tz_str = RASQAL_MALLOC(cstring, TZ_STR_SIZE + 1);
+  tz_str = RASQAL_MALLOC(char*, TZ_STR_SIZE + 1);
   if(!tz_str)
     return NULL;
   
@@ -1327,20 +1403,20 @@ rasqal_xsd_datetime_get_tz_as_counted_string(rasqal_xsd_datetime* dt,
 {
   char* s;
   
-  s = (char*)RASQAL_MALLOC(cstring, TIMEZONE_BUFFER_LEN);
+  s = RASQAL_MALLOC(char*, TIMEZONE_BUFFER_LEN + 1);
   if(!s)
     return NULL;
 
-  if(rasqal_xsd_datetime_timezone_format(dt, s, TIMEZONE_BUFFER_LEN))
+  if(rasqal_xsd_datetime_timezone_format(dt, s, TIMEZONE_BUFFER_LEN + 1) < 0)
     goto failed;
 
   if(len_p)
-    *len_p = (TIMEZONE_BUFFER_LEN - 1); /* \0 is not in the length */
+    *len_p = TIMEZONE_BUFFER_LEN;
   
   return s;
 
   failed:
-  RASQAL_FREE(cstring, s);
+  RASQAL_FREE(char*, s);
   return NULL;
 }
 
@@ -1370,7 +1446,7 @@ static int test_datetime_parser_tostring(const char *in_str, const char *out_exp
     r = strcmp((char*)s, out_expected);
     if(r)
       fprintf(stderr, "input dateTime \"%s\" converted to canonical \"%s\", expected \"%s\"\n", in_str, s, out_expected);
-    RASQAL_FREE(cstring, (void*)s);
+    RASQAL_FREE(char*, s);
   } else
     fprintf(stderr, "input dateTime \"%s\" converted to canonical (null), expected \"%s\"\n", in_str, out_expected);
   return r;
@@ -1386,7 +1462,7 @@ static int test_date_parser_tostring(const char *in_str, const char *out_expecte
     r = strcmp((char*)s, out_expected);
     if(r)
       fprintf(stderr, "input date \"%s\" converted to canonical \"%s\", expected \"%s\"\n", in_str, s, out_expected);
-    RASQAL_FREE(cstring, (void*)s);
+    RASQAL_FREE(char*, s);
   } else
     fprintf(stderr, "input date \"%s\" converted to canonical (null), expected \"%s\"\n", in_str, out_expected);
   return r;
