@@ -29,6 +29,9 @@
 #include <win32_rasqal_config.h>
 #endif
 
+/* for round() prototype */
+#define _ISOC99_SOURCE 1
+
 #include <stdio.h>
 #include <string.h>
 #ifdef HAVE_STDLIB_H
@@ -113,7 +116,8 @@ rasqal_xsd_decimal*
 rasqal_new_xsd_decimal(rasqal_world* world)
 {
   rasqal_xsd_decimal* dec;
-  dec = (rasqal_xsd_decimal*)RASQAL_MALLOC(decimal, sizeof(rasqal_xsd_decimal));
+
+  dec = RASQAL_MALLOC(rasqal_xsd_decimal*, sizeof(*dec));
   if(dec)
     rasqal_xsd_decimal_init(dec);
   return dec;
@@ -178,7 +182,7 @@ static void
 rasqal_xsd_decimal_clear_string(rasqal_xsd_decimal* dec)
 {
   if(dec->string) {
-    RASQAL_FREE(cstring, dec->string);
+    RASQAL_FREE(char*, dec->string);
     dec->string=NULL;
   }
   dec->string_len=0;
@@ -224,7 +228,7 @@ rasqal_xsd_decimal_set_string(rasqal_xsd_decimal* dec, const char* string)
   rasqal_xsd_decimal_clear_string(dec);
 
   len = strlen(string);
-  dec->string = (char*)RASQAL_MALLOC(cstring, len+1);
+  dec->string = RASQAL_MALLOC(char*, len + 1);
   if(!dec->string)
     return 1;
 
@@ -333,6 +337,40 @@ rasqal_xsd_decimal_get_double(rasqal_xsd_decimal* dec)
   return result;
 }
 
+/**
+ * rasqal_xsd_decimal_get_long:
+ * @dec: XSD Decimal
+ * @error_p: pointer to error flag
+ * 
+ * Get an XSD Decimal as a long (may lose precision)
+ * 
+ * Return value: long value or 0 on failure and *error_p is non-0
+ **/
+long
+rasqal_xsd_decimal_get_long(rasqal_xsd_decimal* dec, int* error_p)
+{
+  long result = 0;
+
+#if defined(RASQAL_DECIMAL_C99) || defined(RASQAL_DECIMAL_NONE)
+  result=(long)dec->raw;
+#endif
+#ifdef RASQAL_DECIMAL_MPFR
+  if(!mpfr_fits_slong_p(dec->raw, dec->rounding)) {
+    if(error_p)
+      *error_p = 1;
+  } else
+    result = mpfr_get_si(dec->raw, dec->rounding);
+#endif
+#ifdef RASQAL_DECIMAL_GMP
+  if(!mpf_fits_slong_p(dec->raw)) {
+    if(error_p)
+      *error_p = 1;
+  } else
+    result = mpf_get_si(dec->raw);
+#endif
+
+  return result;
+}
 
 #ifdef RASQAL_DECIMAL_C99
 #define DECIMAL_SNPRINTF snprintf
@@ -394,7 +432,7 @@ rasqal_xsd_decimal_as_string(rasqal_xsd_decimal* dec)
   /* decimal snprintf with no buffer to get buffer length */
   len = DECIMAL_SNPRINTF(NULL, 0, fmt, dec->raw);
 
-  s = (char*)RASQAL_MALLOC(cstring, len + 1);
+  s = RASQAL_MALLOC(char*, len + 1);
   if(!s)
     return NULL;
   
@@ -684,7 +722,7 @@ rasqal_xsd_decimal_abs(rasqal_xsd_decimal* result, rasqal_xsd_decimal* a)
   rasqal_xsd_decimal_clear_string(result);
   
 #if defined(RASQAL_DECIMAL_C99) || defined(RASQAL_DECIMAL_NONE)
-  result->raw = abs(a->raw);
+  result->raw = fabs(a->raw);
 #endif
 #ifdef RASQAL_DECIMAL_MPFR
   mpfr_abs(result->raw, a->raw, result->rounding);
