@@ -38,15 +38,6 @@
 #endif
 #include <stdarg.h>
 
-#ifdef RASQAL_REGEX_PCRE
-#include <pcre.h>
-#endif
-
-#ifdef RASQAL_REGEX_POSIX
-#include <sys/types.h>
-#include <regex.h>
-#endif
-
 #include "rasqal.h"
 #include "rasqal_internal.h"
 
@@ -205,8 +196,8 @@ tidy:
  * Constructor - create a new 3-operand expression.
  * Takes ownership of the operands.
  * 
- * The only operator is:
- * @RASQAL_EXPR_REGEX
+ * The operators are:
+ * #RASQAL_EXPR_REGEX #RASQAL_EXPR_IF #RASQAL_EXPR_SUBSTR
  *
  * Return value: a new #rasqal_expression object or NULL on failure
  **/
@@ -244,6 +235,61 @@ rasqal_new_3op_expression(rasqal_world* world,
 }
 
 
+/**
+ * rasqal_new_4op_expression:
+ * @world: rasqal_world object
+ * @op: Expression operator
+ * @arg1: Operand 1 
+ * @arg2: Operand 2
+ * @arg3: Operand 3
+ * @arg4: Operand 4 (may be NULL)
+ * 
+ * Constructor - create a new 4-operand expression.
+ * Takes ownership of the operands.
+ * 
+ * The operators are:
+ * #RASQAL_EXPR_REPLACE
+ *
+ * Return value: a new #rasqal_expression object or NULL on failure
+ **/
+rasqal_expression*
+rasqal_new_4op_expression(rasqal_world* world,
+                          rasqal_op op,
+                          rasqal_expression* arg1, 
+                          rasqal_expression* arg2,
+                          rasqal_expression* arg3,
+                          rasqal_expression* arg4)
+{
+  rasqal_expression* e = NULL;
+
+  if(!world || !arg1 || !arg2 || !arg3) /* arg4 may be NULL */
+    goto tidy;
+
+  e = RASQAL_CALLOC(rasqal_expression*, 1, sizeof(*e));
+  if(e) {
+    e->usage = 1;
+    e->world = world;
+    e->op = op;
+    e->arg1 = arg1; arg1 = NULL;
+    e->arg2 = arg2; arg2 = NULL;
+    e->arg3 = arg3; arg3 = NULL;
+    e->arg4 = arg4; arg4 = NULL;
+  }
+
+  tidy:
+  if(arg1)
+    rasqal_free_expression(arg1);
+  if(arg2)
+    rasqal_free_expression(arg2);
+  if(arg3)
+    rasqal_free_expression(arg3);
+  if(arg4)
+    rasqal_free_expression(arg4);
+
+  return e;
+}
+
+
 /**
  * rasqal_new_string_op_expression:
  * @world: rasqal_world object
@@ -622,6 +668,8 @@ rasqal_expression_clear(rasqal_expression* e)
     case RASQAL_EXPR_SAMETERM:
     case RASQAL_EXPR_STRLANG:
     case RASQAL_EXPR_STRDT:
+    case RASQAL_EXPR_STRBEFORE:
+    case RASQAL_EXPR_STRAFTER:
       rasqal_free_expression(e->arg1);
       rasqal_free_expression(e->arg2);
       break;
@@ -632,10 +680,13 @@ rasqal_expression_clear(rasqal_expression* e)
     case RASQAL_EXPR_STRENDS:
     case RASQAL_EXPR_CONTAINS:
     case RASQAL_EXPR_SUBSTR:
+    case RASQAL_EXPR_REPLACE:
       rasqal_free_expression(e->arg1);
       rasqal_free_expression(e->arg2);
       if(e->arg3)
         rasqal_free_expression(e->arg3);
+      if(e->arg4)
+        rasqal_free_expression(e->arg4);
       break;
 
     case RASQAL_EXPR_TILDE:
@@ -836,6 +887,8 @@ rasqal_expression_visit(rasqal_expression* e,
     case RASQAL_EXPR_STRSTARTS:
     case RASQAL_EXPR_STRENDS:
     case RASQAL_EXPR_CONTAINS:
+    case RASQAL_EXPR_STRBEFORE:
+    case RASQAL_EXPR_STRAFTER:
       return rasqal_expression_visit(e->arg1, fn, user_data) ||
              rasqal_expression_visit(e->arg2, fn, user_data);
       break;
@@ -848,6 +901,13 @@ rasqal_expression_visit(rasqal_expression* e,
              (e->arg3 && rasqal_expression_visit(e->arg3, fn, user_data));
       break;
 
+    case RASQAL_EXPR_REPLACE:
+      return rasqal_expression_visit(e->arg1, fn, user_data) ||
+             rasqal_expression_visit(e->arg2, fn, user_data) ||
+             rasqal_expression_visit(e->arg3, fn, user_data) ||
+             (e->arg4 && rasqal_expression_visit(e->arg4, fn, user_data));
+      break;
+
     case RASQAL_EXPR_TILDE:
     case RASQAL_EXPR_BANG:
     case RASQAL_EXPR_UMINUS:
@@ -1042,7 +1102,10 @@ static const char* const rasqal_op_labels[RASQAL_EXPR_LAST+1]={
   "sha224",
   "sha256",
   "sha384",
-  "sha512"
+  "sha512",
+  "strbefore",
+  "strafter",
+  "replace"
 };
 
 
@@ -1153,18 +1216,28 @@ rasqal_expression_write(rasqal_expression* e, raptor_iostream* iostr)
     case RASQAL_EXPR_STRENDS:
     case RASQAL_EXPR_SUBSTR:
     case RASQAL_EXPR_CONTAINS:
+    case RASQAL_EXPR_STRBEFORE:
+    case RASQAL_EXPR_STRAFTER:
+    case RASQAL_EXPR_REPLACE:
       raptor_iostream_counted_string_write("op ", 3, iostr);
       rasqal_expression_write_op(e, iostr);
       raptor_iostream_write_byte('(', iostr);
       rasqal_expression_write(e->arg1, iostr);
       raptor_iostream_counted_string_write(", ", 2, iostr);
       rasqal_expression_write(e->arg2, iostr);
-      /* There are three 3-op expressions - both handled here */
+
+      /* There are four 3+ arg expressions - all handled here */
       if((e->op == RASQAL_EXPR_REGEX || e->op == RASQAL_EXPR_IF ||
-          e->op == RASQAL_EXPR_SUBSTR) && e->arg3) {
+          e->op == RASQAL_EXPR_SUBSTR || e->op == RASQAL_EXPR_REPLACE)
+         && e->arg3) {
         raptor_iostream_counted_string_write(", ", 2, iostr);
         rasqal_expression_write(e->arg3, iostr);
       }
+      /* One 3 or 4 arg expression */
+      if((e->op == RASQAL_EXPR_REPLACE) && e->arg4) {
+        raptor_iostream_counted_string_write(", ", 2, iostr);
+        rasqal_expression_write(e->arg4, iostr);
+      }
       raptor_iostream_write_byte(')', iostr);
       break;
 
@@ -1368,18 +1441,28 @@ rasqal_expression_print(rasqal_expression* e, FILE* fh)
     case RASQAL_EXPR_STRENDS:
     case RASQAL_EXPR_CONTAINS:
     case RASQAL_EXPR_SUBSTR:
+    case RASQAL_EXPR_STRBEFORE:
+    case RASQAL_EXPR_STRAFTER:
+    case RASQAL_EXPR_REPLACE:
       fputs("op ", fh);
       rasqal_expression_print_op(e, fh);
       fputc('(', fh);
       rasqal_expression_print(e->arg1, fh);
       fputs(", ", fh);
       rasqal_expression_print(e->arg2, fh);
-      /* There are three 3-op expressions - both handled here */
+
+      /* There are four 3+ arg expressions - all handled here */
       if((e->op == RASQAL_EXPR_REGEX || e->op == RASQAL_EXPR_IF ||
-          e->op == RASQAL_EXPR_SUBSTR) && e->arg3) {
+          e->op == RASQAL_EXPR_SUBSTR || e->op == RASQAL_EXPR_REPLACE)
+         && e->arg3) {
         fputs(", ", fh);
         rasqal_expression_print(e->arg3, fh);
       }
+      /* One 3 or 4 arg expression */
+      if((e->op == RASQAL_EXPR_REPLACE) && e->arg4) {
+        fputs(", ", fh);
+        rasqal_expression_print(e->arg4, fh);
+      }
       fputc(')', fh);
       break;
 
@@ -1585,6 +1668,8 @@ rasqal_expression_is_constant(rasqal_expression* e)
     case RASQAL_EXPR_STRSTARTS:
     case RASQAL_EXPR_STRENDS:
     case RASQAL_EXPR_CONTAINS:
+    case RASQAL_EXPR_STRBEFORE:
+    case RASQAL_EXPR_STRAFTER:
       result = rasqal_expression_is_constant(e->arg1) &&
                rasqal_expression_is_constant(e->arg2);
       break;
@@ -1597,6 +1682,13 @@ rasqal_expression_is_constant(rasqal_expression* e)
                (e->arg3 && rasqal_expression_is_constant(e->arg3));
       break;
 
+    case RASQAL_EXPR_REPLACE:
+      result = rasqal_expression_is_constant(e->arg1) &&
+               rasqal_expression_is_constant(e->arg2) &&
+               rasqal_expression_is_constant(e->arg3) &&
+               (e->arg4 && rasqal_expression_is_constant(e->arg4));
+      break;
+
     case RASQAL_EXPR_STR_MATCH:
     case RASQAL_EXPR_STR_NMATCH:
       result = rasqal_expression_is_constant(e->arg1) &&
@@ -1932,6 +2024,8 @@ rasqal_expression_compare(rasqal_expression* e1, rasqal_expression* e2,
     case RASQAL_EXPR_STRENDS:
     case RASQAL_EXPR_CONTAINS:
     case RASQAL_EXPR_SUBSTR:
+    case RASQAL_EXPR_STRBEFORE:
+    case RASQAL_EXPR_STRAFTER:
       rc = rasqal_expression_compare(e1->arg1, e2->arg1, flags, error_p);
       if(rc)
         return rc;
@@ -1940,13 +2034,30 @@ rasqal_expression_compare(rasqal_expression* e1, rasqal_expression* e2,
       if(rc)
         return rc;
       
-      /* There are two 3-op expressions - both handled here */
+      /* There are three 3-op expressions - both handled here */
       if(e1->op == RASQAL_EXPR_REGEX || e1->op == RASQAL_EXPR_IF ||
          e1->op == RASQAL_EXPR_SUBSTR)
         rc = rasqal_expression_compare(e1->arg3, e2->arg3, flags, error_p);
 
       break;
 
+    case RASQAL_EXPR_REPLACE:
+      /* 3 or 4 args */
+      rc = rasqal_expression_compare(e1->arg1, e2->arg1, flags, error_p);
+      if(rc)
+        return rc;
+      
+      rc = rasqal_expression_compare(e1->arg2, e2->arg2, flags, error_p);
+      if(rc)
+        return rc;
+      
+      rc = rasqal_expression_compare(e1->arg3, e2->arg3, flags, error_p);
+      if(rc)
+        return rc;
+
+      rc = rasqal_expression_compare(e1->arg4, e2->arg4, flags, error_p);
+      break;
+
     case RASQAL_EXPR_STR_MATCH:
     case RASQAL_EXPR_STR_NMATCH:
       rc = rasqal_expression_compare(e1->arg1, e2->arg1, flags, error_p);
@@ -2250,6 +2361,12 @@ rasqal_new_evaluation_context(rasqal_world* world,
   eval_context->locator = locator;
   eval_context->flags = flags;
 
+  eval_context->random = rasqal_new_random(world);
+  if(!eval_context->random) {
+    RASQAL_FREE(rasqal_evaluation_context*, eval_context);
+    eval_context = NULL;
+  }
+
   return eval_context;
 }
 
@@ -2266,11 +2383,14 @@ rasqal_free_evaluation_context(rasqal_evaluation_context* eval_context)
 {
   if(!eval_context)
     return;
-  
+
   if(eval_context->base_uri)
     raptor_free_uri(eval_context->base_uri);
 
-  RASQAL_FREE(rasqal_evaluation_context, eval_context);
+  if(eval_context->random)
+    rasqal_free_random(eval_context->random);
+
+  RASQAL_FREE(rasqal_evaluation_context*, eval_context);
 }
 
 
@@ -2315,13 +2435,7 @@ rasqal_evaluation_context_set_rand_seed(rasqal_evaluation_context* eval_context,
   
   RASQAL_ASSERT_OBJECT_POINTER_RETURN_VALUE(eval_context, rasqal_evaluation_context, 1);
 
-#ifdef HAVE_RAND_R
-  eval_context->seed = seed;
-#else
-  srand(seed);
-#endif
-
-  return 0;
+  return rasqal_random_seed(eval_context->random, seed);
 }
 
 
