@@ -237,11 +237,7 @@ rasqal_expression_evaluate_rand(rasqal_expression *e,
   rasqal_world* world = eval_context->world;
   double d;
   
-#ifdef HAVE_RAND_R
-  d = (0.0 + rand_r(&eval_context->seed)) / (double)(RAND_MAX + 1.0);
-#else
-  d = (0.0 + rand())  / (double)(RAND_MAX + 1.0);
-#endif
+  d = rasqal_random_drand(eval_context->random);
 
   return rasqal_new_double_literal(world, d);
 }
