@@ -47,7 +47,7 @@ rasqal_iostream_write_html_literal(rasqal_world* world,
                                    rasqal_literal* l)
 {
   if(!l) {
-    raptor_iostream_counted_string_write("<span class=\"unbound\">", 17, iostr);
+    raptor_iostream_counted_string_write("<span class=\"unbound\">", 22, iostr);
     raptor_iostream_counted_string_write("unbound", 7, iostr);
   } else switch(l->type) {
     const unsigned char* str;
@@ -69,7 +69,18 @@ rasqal_iostream_write_html_literal(rasqal_world* world,
       raptor_xml_escape_string_write(l->string, l->string_len, 0, iostr);
       break;
 
+    case RASQAL_LITERAL_XSD_STRING:
+    case RASQAL_LITERAL_BOOLEAN:
+    case RASQAL_LITERAL_INTEGER:
+    case RASQAL_LITERAL_DOUBLE:
     case RASQAL_LITERAL_STRING:
+    case RASQAL_LITERAL_PATTERN:
+    case RASQAL_LITERAL_QNAME:
+    case RASQAL_LITERAL_FLOAT:
+    case RASQAL_LITERAL_DECIMAL:
+    case RASQAL_LITERAL_DATETIME:
+    case RASQAL_LITERAL_UDT:
+    case RASQAL_LITERAL_INTEGER_SUBTYPE:
       raptor_iostream_counted_string_write("<span class=\"literal\">", 22, iostr);
       raptor_iostream_counted_string_write("<span class=\"value\"", 19, iostr);
       if(l->language) {
@@ -91,24 +102,15 @@ rasqal_iostream_write_html_literal(rasqal_world* world,
       }
       break;
 
-    case RASQAL_LITERAL_PATTERN:
-    case RASQAL_LITERAL_QNAME:
-    case RASQAL_LITERAL_INTEGER:
-    case RASQAL_LITERAL_XSD_STRING:
-    case RASQAL_LITERAL_BOOLEAN:
-    case RASQAL_LITERAL_DOUBLE:
-    case RASQAL_LITERAL_FLOAT:
     case RASQAL_LITERAL_VARIABLE:
-    case RASQAL_LITERAL_DECIMAL:
-    case RASQAL_LITERAL_DATETIME:
-    case RASQAL_LITERAL_UDT:
-    case RASQAL_LITERAL_INTEGER_SUBTYPE:
+      return rasqal_iostream_write_html_literal(world, iostr, l->value.variable->value);
 
     case RASQAL_LITERAL_UNKNOWN:
     default:
       rasqal_log_error_simple(world, RAPTOR_LOG_LEVEL_ERROR, NULL,
                               "Cannot turn literal type %d into HTML", 
                               l->type);
+      return 1;
   }
 
   raptor_iostream_counted_string_write("</span>", 7, iostr);
@@ -130,7 +132,7 @@ rasqal_query_results_write_html_bindings(raptor_iostream *iostr,
   raptor_iostream_counted_string_write("    <tr>\n", 9, iostr);
   for(i = 0; 1; i++) {
     const unsigned char *name;
-    int len;
+    size_t len;
     
     name = rasqal_query_results_get_binding_name(results, i);
     if(!name)
