@@ -115,6 +115,8 @@ typedef struct
   /* Variables table allocated for variables in the result set */
   rasqal_variables_table* vars_table;
   int variables_count;
+
+  unsigned int flags;
 } rasqal_rowsource_rdf_context;
 
 
@@ -463,6 +465,11 @@ rasqal_rowsource_rdf_finish(rasqal_rowsource* rowsource, void *user_data)
   if(con->ds)
     rasqal_free_dataset(con->ds);
 
+  if(con->flags) {
+    if(con->iostr)
+      raptor_free_iostream(con->iostr);
+  }
+
   RASQAL_FREE(rasqal_rowsource_rdf_context, con);
 
   return 0;
@@ -574,7 +581,7 @@ rasqal_rowsource_rdf_process(rasqal_rowsource_rdf_context* con)
         unsigned char* var_name;
         rasqal_variable *v;
 
-        var_name = (unsigned char*)RASQAL_MALLOC(cstring, len + 1);
+        var_name = RASQAL_MALLOC(unsigned char*, len + 1);
         memcpy(var_name, name, len + 1);
 
         v = rasqal_variables_table_add(con->vars_table,
@@ -744,7 +751,7 @@ static const rasqal_rowsource_handler rasqal_rowsource_rdf_handler={
   /* .read_row = */ rasqal_rowsource_rdf_read_row,
   /* .read_all_rows = */ NULL,
   /* .reset = */ NULL,
-  /* .set_preserve = */ NULL,
+  /* .set_requirements = */ NULL,
   /* .get_inner_rowsource = */ NULL,
   /* .set_origin = */ NULL,
 };
@@ -767,11 +774,12 @@ rasqal_query_results_rdf_get_rowsource(rasqal_query_results_formatter* formatter
                                        rasqal_world *world,
                                        rasqal_variables_table* vars_table,
                                        raptor_iostream *iostr,
-                                       raptor_uri *base_uri)
+                                       raptor_uri *base_uri,
+                                       unsigned int flags)
 {
   rasqal_rowsource_rdf_context* con;
   
-  con = (rasqal_rowsource_rdf_context*)RASQAL_CALLOC(rasqal_rowsource_rdf_context, 1, sizeof(*con));
+  con = RASQAL_CALLOC(rasqal_rowsource_rdf_context*, 1, sizeof(*con));
   if(!con)
     return NULL;
 
@@ -779,6 +787,7 @@ rasqal_query_results_rdf_get_rowsource(rasqal_query_results_formatter* formatter
   con->formatter = formatter;
   con->base_uri = base_uri ? raptor_uri_copy(base_uri) : NULL;
   con->iostr = iostr;
+  con->flags = flags;
 
   con->results_sequence = raptor_new_sequence((raptor_data_free_handler)rasqal_free_row, (raptor_data_print_handler)rasqal_row_print);
 
