@@ -116,7 +116,7 @@ rasqal_new_world(void)
 {
   rasqal_world* world;
 
-  world = RASQAL_CALLOC(rasqal_world*, sizeof(rasqal_world), 1);
+  world = RASQAL_CALLOC(rasqal_world*, 1, sizeof(*world));
   if(!world)
     return NULL;
   
@@ -383,7 +383,7 @@ rasqal_query_language_register_factory(rasqal_world *world,
 
 #if defined(RASQAL_DEBUG) && RASQAL_DEBUG > 1
   RASQAL_DEBUG3("Registered query language %s with context size %d\n",
-                query->names[0], query->context_length);
+                query->desc.names[0], query->context_length);
 #endif
 
   return query;
@@ -592,7 +592,14 @@ rasqal_log_error_varargs(rasqal_world* world, raptor_log_level level,
   if(level == RAPTOR_LOG_LEVEL_NONE)
     return;
 
-  buffer=raptor_vsnprintf(message, arguments);
+#if RAPTOR_VERSION >= 20005
+  buffer = NULL;
+  if(raptor_vasprintf(&buffer, message, arguments) < 0)
+    buffer = NULL;
+#else
+  buffer = raptor_vsnprintf(message, arguments);
+#endif
+
   if(!buffer) {
     if(locator) {
       raptor_locator_print(locator, stderr);
