@@ -62,8 +62,7 @@ rasqal_new_graph_pattern(rasqal_query* query,
 
   RASQAL_ASSERT_OBJECT_POINTER_RETURN_VALUE(query, rasqal_query, NULL);
   
-  gp = (rasqal_graph_pattern*)RASQAL_CALLOC(rasqal_graph_pattern, 1, 
-                                            sizeof(*gp));
+  gp = RASQAL_CALLOC(rasqal_graph_pattern*, 1, sizeof(*gp));
   if(!gp)
     return NULL;
 
@@ -220,16 +219,27 @@ rasqal_new_let_graph_pattern(rasqal_query *query,
  * rasqal_new_select_graph_pattern:
  * @query: #rasqal_graph_pattern query object
  * @projection: projection object
+ * @data_graphs: sequence of #rasqal_data_graph (or NULL)
  * @where: WHERE graph pattern
- * @modifiers: solution modifier
+ * @modifier: solution modifier
  *
  * INTERNAL - Create a new SELECT graph pattern
- * 
+ *
+ * The @projection, @data_graphs, @where and @modifier all become owned
+ * by the new graph pattern object.
+ *
+ * Roughly corresponds to:
+ *   SELECT DISTINCT [in @projection] {@projection}
+ *   FROM @data_graphs
+ *   WHERE @where
+ *   GROUP BY (HAVING)/ORDER BY [in @modifier]
+ *
  * Return value: a new #rasqal_graph_pattern object or NULL on failure
  **/
 rasqal_graph_pattern*
 rasqal_new_select_graph_pattern(rasqal_query *query,
                                 rasqal_projection* projection,
+                                raptor_sequence* data_graphs,
                                 rasqal_graph_pattern* where,
                                 rasqal_solution_modifier* modifier)
 {
@@ -252,6 +262,7 @@ rasqal_new_select_graph_pattern(rasqal_query *query,
   }
 
   gp->projection = projection;
+  gp->data_graphs = data_graphs;
   gp->modifier = modifier;
   
   if(rasqal_graph_pattern_add_sub_graph_pattern(gp, where)) {
@@ -325,6 +336,9 @@ rasqal_free_graph_pattern(rasqal_graph_pattern* gp)
   if(gp->modifier)
     rasqal_free_solution_modifier(gp->modifier);
 
+  if(gp->data_graphs)
+    raptor_free_sequence(gp->data_graphs);
+  
   RASQAL_FREE(rasqal_graph_pattern, gp);
 }
 
@@ -505,10 +519,24 @@ rasqal_graph_pattern_write_internal(rasqal_graph_pattern* gp,
   if(indent >= 0)
     indent += 2;
 
+  if(gp->silent) {
+    raptor_iostream_counted_string_write("silent", 6, iostr);
+    pending_nl = 1;
+  }
+
   if(gp->triples) {
     int size = gp->end_column - gp->start_column + 1;
     int i;
 
+    if(pending_nl) {
+      raptor_iostream_counted_string_write(" ,", 2, iostr);
+
+      if(indent >= 0) {
+        raptor_iostream_write_byte('\n', iostr);
+        rasqal_graph_pattern_write_indent(iostr, indent);
+      }
+    }
+
     raptor_iostream_counted_string_write("over ", 5, iostr);
     rasqal_graph_pattern_write_plurals(iostr, "triple", size);
     raptor_iostream_write_byte('[', iostr);
@@ -964,7 +992,6 @@ rasqal_graph_pattern_set_origin(rasqal_graph_pattern* graph_pattern,
  * rasqal_new_basic_graph_pattern_from_formula:
  * @query: #rasqal_graph_pattern query object
  * @formula: triples sequence containing the graph pattern
- * @op: enum #rasqal_graph_pattern_operator operator
  *
  * INTERNAL - Create a new graph pattern object over a formula. This function
  * frees the formula passed in.
