@@ -1171,7 +1171,10 @@ rasqal_literal* rasqal_expression_evaluate_encode_for_uri(rasqal_expression *e,
 rasqal_literal* rasqal_expression_evaluate_concat(rasqal_expression *e, rasqal_evaluation_context *eval_context, int *error_p);
 rasqal_literal* rasqal_expression_evaluate_langmatches(rasqal_expression *e, rasqal_evaluation_context *eval_context, int *error_p);
 rasqal_literal* rasqal_expression_evaluate_strmatch(rasqal_expression *e, rasqal_evaluation_context *eval_context, int *error_p);
-  
+rasqal_literal* rasqal_expression_evaluate_strbefore(rasqal_expression *e, rasqal_evaluation_context *eval_context, int *error_p);
+rasqal_literal* rasqal_expression_evaluate_strafter(rasqal_expression *e, rasqal_evaluation_context *eval_context, int *error_p);
+rasqal_literal* rasqal_expression_evaluate_replace(rasqal_expression *e, rasqal_evaluation_context *eval_context, int *error_p);
+
 
 /* strcasecmp.c */
 #ifdef HAVE_STRCASECMP
@@ -1291,6 +1294,7 @@ void rasqal_query_results_remove_query_reference(rasqal_query_results* query_res
 rasqal_variables_table* rasqal_query_results_get_variables_table(rasqal_query_results* query_results);
 rasqal_row* rasqal_query_results_get_current_row(rasqal_query_results* query_results);
 rasqal_world* rasqal_query_results_get_world(rasqal_query_results* query_results);
+int rasqal_query_results_sort(rasqal_query_results* query_result, raptor_data_compare_handler compare);
 
 
 /* rasqal_query_write.c */
@@ -1334,6 +1338,7 @@ int rasqal_row_set_order_size(rasqal_row *row, int order_size);
 int rasqal_row_expand_size(rasqal_row *row, int size);
 int rasqal_row_bind_variables(rasqal_row* row, rasqal_variables_table* vars_table);
 raptor_sequence* rasqal_row_sequence_copy(raptor_sequence *seq);
+int rasqal_row_compare(const void *a, const void *b);
 
 /* rasqal_row_compatible.c */
 rasqal_row_compatible* rasqal_new_row_compatible(rasqal_variables_table* vt, rasqal_rowsource *first_rowsource, rasqal_rowsource *second_rowsource);
@@ -1735,6 +1740,10 @@ raptor_sequence* rasqal_projection_get_variables_sequence(rasqal_projection* pro
 int rasqal_projection_add_variable(rasqal_projection* projection, rasqal_variable* var);
 
 
+/* rasqal_regex.c */
+int rasqal_regex_match(rasqal_world* world, raptor_locator* locator, const char* pattern, const char* regex_flags, const char* subject, size_t subject_len);
+char* rasqal_regex_replace(rasqal_world* world, raptor_locator* locator, const char* pattern, const char* regex_flags, const char* subject, size_t subject_len, const char* replace, size_t replace_len, size_t* result_len_p);
+
 /* rasqal_solution_modifier.c */
 rasqal_solution_modifier* rasqal_new_solution_modifier(rasqal_query* query, raptor_sequence* order_conditions, raptor_sequence* group_conditions, raptor_sequence* having_conditions, int limit, int offset);
 void rasqal_free_solution_modifier(rasqal_solution_modifier* sm);
@@ -1743,7 +1752,41 @@ void rasqal_free_solution_modifier(rasqal_solution_modifier* sm);
 int rasqal_triples_sequence_set_origin(raptor_sequence* dest_seq, raptor_sequence* src_seq, rasqal_literal* origin);
 
 /* rasqal_random.c */
-unsigned int rasqal_random_get_system_seed(rasqal_world* world);
+/**
+ * RASQAL_RANDOM_STATE_SIZE:
+ *
+ * Size of BSD random state
+ * 
+ * "With 256 bytes of state information, the period of the random
+ * number generator is greater than 2**69 , which should be
+ * sufficient for most purposes." - BSD random(3) man page
+ */
+#define RASQAL_RANDOM_STATE_SIZE 256
+
+
+/**
+ * rasqal_random:
+ * @world: world object
+ * @seed: used for rand_r() (if available) or srand()
+ * @state: used for BSD initstate(), setstate() and random() (if available)
+ * @data: internal to random algorithm
+ *
+ * A class providing a random number generator
+ *
+ */
+struct rasqal_random_s {
+  rasqal_world* world;
+  unsigned int seed;
+  char state[RASQAL_RANDOM_STATE_SIZE];
+  void* data;
+};
+
+unsigned int rasqal_random_get_system_seed(rasqal_world *world);
+rasqal_random* rasqal_new_random(rasqal_world *world);
+void rasqal_free_random(rasqal_random *random_object);
+int rasqal_random_seed(rasqal_random *random_object, unsigned int seed);
+int rasqal_random_irand(rasqal_random *random_object);
+double rasqal_random_drand(rasqal_random *random_object);
 
 /*
  * rasqal_digest_type:
@@ -1778,6 +1821,8 @@ int rasqal_digest_md5_buffer(const unsigned char *output,
                              const unsigned char *input, size_t len);
 #endif
 
+/* snprint.c */
+int rasqal_format_integer(char* buffer, size_t bufsize, int integer, int width, char padding);
 
 /* end of RASQAL_INTERNAL */
 #endif
