@@ -2,7 +2,7 @@
  *
  * rasqal_map.c - Rasqal simple Key:Value Map with duplicates allowed
  *
- * Copyright (C) 2005-2008, David Beckett http://www.dajobe.org/
+ * Copyright (C) 2005-2010, David Beckett http://www.dajobe.org/
  * Copyright (C) 2005-2005, University of Bristol, UK http://www.bristol.ac.uk/
  * 
  * This package is Free Software and part of Redland http://librdf.org/
@@ -54,15 +54,11 @@ struct rasqal_map_s {
   struct rasqal_map_node_s* root;
   rasqal_compare_fn* compare;
   void *compare_user_data;
-  rasqal_compare_free_user_data_fn* free_compare_data;
-  rasqal_kv_free_fn* free;
-#ifdef HAVE_RAPTOR2_API
+  raptor_data_free_handler free_compare_data;
+  raptor_data_free_handler free_key;
+  raptor_data_free_handler free_value;
   raptor_data_print_handler print_key;
   raptor_data_print_handler print_value;
-#else
-  raptor_sequence_print_handler* print_key;
-  raptor_sequence_print_handler* print_value;
-#endif
   int allow_duplicates;
 };
 
@@ -73,27 +69,35 @@ static rasqal_map_node*
 rasqal_new_map_node(rasqal_map* map, void *key, void *value)
 {
   rasqal_map_node *node;
-  node=(rasqal_map_node*)RASQAL_CALLOC(rasqal_map_node, 1, sizeof(rasqal_map_node));
+
+  node = RASQAL_CALLOC(rasqal_map_node*, 1, sizeof(*node));
   if(!node)
     return NULL;
-  node->map=map;
-  node->key=key;
-  node->value=value;
+
+  node->map = map;
+  node->key = key;
+  node->value = value;
   return node;
 }
 
 
 static void
-rasqal_free_map_node(rasqal_map_node *node, rasqal_kv_free_fn* free_fn) 
+rasqal_free_map_node(rasqal_map* map, rasqal_map_node *node) 
 {
-  RASQAL_ASSERT_OBJECT_POINTER_RETURN(node, rasqal_map_node);
+  if(!node)
+    return;
   
   if(node->prev)
-    rasqal_free_map_node(node->prev, free_fn);
-  if(node->next)
-    rasqal_free_map_node(node->next, free_fn);
+    rasqal_free_map_node(map, node->prev);
 
-  free_fn(node->key, node->value);
+  if(node->next)
+    rasqal_free_map_node(map, node->next);
+
+  if(map->free_key)
+    map->free_key(node->key);
+
+  if(map->free_value)
+    map->free_value(node->value);
 
   RASQAL_FREE(rasqal_map_node, node);
 }
@@ -113,28 +117,19 @@ rasqal_free_map_node(rasqal_map_node *node, rasqal_kv_free_fn* free_fn)
  * 
  * Return value: a new #rasqal_map or NULL on failure
  **/
-#ifdef HAVE_RAPTOR2_API
 rasqal_map*
 rasqal_new_map(rasqal_compare_fn* compare_fn,
                void *compare_user_data,
-               rasqal_compare_free_user_data_fn* free_compare_data_fn,
-               rasqal_kv_free_fn* free_fn,
+               raptor_data_free_handler free_compare_data_fn,
+               raptor_data_free_handler free_key_fn,
+               raptor_data_free_handler free_value_fn,
                raptor_data_print_handler print_key_fn,
                raptor_data_print_handler print_value_fn,
                int flags)
-#else
-rasqal_map*
-rasqal_new_map(rasqal_compare_fn* compare_fn,
-               void *compare_user_data,
-               rasqal_compare_free_user_data_fn* free_compare_data_fn,
-               rasqal_kv_free_fn* free_fn,
-               raptor_sequence_print_handler* print_key_fn,
-               raptor_sequence_print_handler* print_value_fn,
-               int flags)
-#endif
 {
   rasqal_map *map;
-  map = (rasqal_map*)RASQAL_CALLOC(rasqal_map, 1, sizeof(rasqal_map));
+
+  map = RASQAL_CALLOC(rasqal_map*, 1, sizeof(*map));
   if(!map) {
     if(free_compare_data_fn)
       free_compare_data_fn(compare_user_data);
@@ -144,7 +139,8 @@ rasqal_new_map(rasqal_compare_fn* compare_fn,
   map->compare = compare_fn;
   map->compare_user_data = compare_user_data;
   map->free_compare_data = free_compare_data_fn;
-  map->free = free_fn;
+  map->free_key = free_key_fn;
+  map->free_value = free_value_fn;
   map->print_key = print_key_fn;
   map->print_value = print_value_fn;
   map->allow_duplicates = flags;
@@ -167,7 +163,7 @@ rasqal_free_map(rasqal_map *map)
     return;
   
   if(map->root)
-    rasqal_free_map_node(map->root, map->free);
+    rasqal_free_map_node(map, map->root);
 
   if(map->free_compare_data)
     map->free_compare_data(map->compare_user_data);
@@ -186,6 +182,7 @@ rasqal_map_node_add_kv(rasqal_map_node* node, void *key, void *value)
   if(result < 0) {
     if(node->prev)
       return rasqal_map_node_add_kv(node->prev, key, value);
+
     node->prev = rasqal_new_map_node(map, key, value);
     return node->prev ? 0 : -1;
   } else if(!result) {
@@ -205,6 +202,39 @@ rasqal_map_node_add_kv(rasqal_map_node* node, void *key, void *value)
 }
 
 
+static rasqal_map_node*
+rasqal_map_search_internal(rasqal_map* map, rasqal_map_node* node,
+                           const void* key)
+{
+  if(node) {
+    int cmp = map->compare(map->compare_user_data, key, node->key);
+
+    if(cmp > 0)
+      return rasqal_map_search_internal(map, node->next, key);
+    else if(cmp < 0)
+      return rasqal_map_search_internal(map, node->prev, key);
+
+    /* found */
+    return node;
+  }
+
+  /* otherwise not found */
+  return NULL;
+}
+
+
+void*
+rasqal_map_search(rasqal_map* map, const void* key)
+{
+  rasqal_map_node* node;
+
+  node = rasqal_map_search_internal(map, map->root, key);
+
+  return node ? node->value : NULL;
+}
+
+
+
 /**
  * rasqal_map_add_kv:
  * @map: #rasqal_map adding to
@@ -219,7 +249,7 @@ int
 rasqal_map_add_kv(rasqal_map* map, void* key, void *value)
 {
   if(!map->root) {
-    map->root=rasqal_new_map_node(map, key, value);
+    map->root = rasqal_new_map_node(map, key, value);
     return map->root ? 0 : -1;
   }
   
@@ -235,7 +265,7 @@ static void
 rasqal_map_node_write_indent(FILE *fh, int indent) 
 {
   while(indent > 0) {
-    int sp=(indent > SPACES_LENGTH) ? SPACES_LENGTH : indent;
+    int sp = (indent > SPACES_LENGTH) ? SPACES_LENGTH : indent;
     (void)fwrite(rasqal_map_node_spaces, sizeof(char), sp, fh);
     indent -= sp;
   }
@@ -249,7 +279,9 @@ rasqal_map_node_visit(rasqal_map_node* node,
 {
   if(node->prev)
     rasqal_map_node_visit(node->prev, fn, user_data);
+
   fn(node->key, node->value, user_data);
+
   if(node->next)
     rasqal_map_node_visit(node->next, fn, user_data);
 }
@@ -287,8 +319,8 @@ rasqal_map_node_print_visit(void *key, void *value, void *user_data)
   FILE* fh;
   int indent;
   
-  fh=pi->fh;
-  indent=pi->indent;
+  fh = pi->fh;
+  indent = pi->indent;
 
   rasqal_map_node_write_indent(fh, indent);
   fputs("{key: ", fh);
@@ -325,11 +357,12 @@ int
 rasqal_map_print(rasqal_map* map, FILE* fh)
 {
   fprintf(fh, "map duplicates=%s {\n", map->allow_duplicates ? "yes" : "no");
+
   if(map->root) {
     struct print_info pi;
-    pi.map=map;
-    pi.fh=fh;
-    pi.indent=2;
+    pi.map = map;
+    pi.fh = fh;
+    pi.indent = 2;
     rasqal_map_visit(map, rasqal_map_node_print_visit, &pi);
   }
   
