@@ -73,13 +73,6 @@
 
 #define DEBUG_FH stderr
 
-#if 0
-#undef RASQAL_NO_GP_MERGE
-#else
-#define RASQAL_NO_GP_MERGE 1
-#endif
-
-
 static int rasqal_query_add_query_result(rasqal_query* query, rasqal_query_results* query_results);
 
 
@@ -114,7 +107,7 @@ rasqal_new_query(rasqal_world *world, const char *name,
   if(!factory)
     return NULL;
 
-  query = (rasqal_query*)RASQAL_CALLOC(rasqal_query, 1, sizeof(rasqal_query));
+  query = RASQAL_CALLOC(rasqal_query*, 1, sizeof(*query));
   if(!query)
     return NULL;
   
@@ -127,8 +120,7 @@ rasqal_new_query(rasqal_world *world, const char *name,
 
   query->genid_counter = 1;
 
-  query->context = (char*)RASQAL_CALLOC(rasqal_query_context, 1,
-                                        factory->context_length);
+  query->context = RASQAL_CALLOC(void*, 1, factory->context_length);
   if(!query->context)
     goto tidy;
   
@@ -205,12 +197,11 @@ rasqal_free_query(rasqal_query* query)
     raptor_free_uri(query->base_uri);
 
   if(query->query_string)
-    RASQAL_FREE(cstring, query->query_string);
+    RASQAL_FREE(char*, query->query_string);
 
   if(query->data_graphs)
     raptor_free_sequence(query->data_graphs);
-  if(query->selects)
-    raptor_free_sequence(query->selects);
+
   if(query->describes)
     raptor_free_sequence(query->describes);
 
@@ -225,11 +216,11 @@ rasqal_free_query(rasqal_query* query)
   if(query->results)
     raptor_free_sequence(query->results);
 
-  if(query->variables_bound_in)
-    RASQAL_FREE(intarray, query->variables_bound_in);
+  if(query->triples_use_map)
+    RASQAL_FREE(shortarray, query->triples_use_map);
 
   if(query->variables_use_map)
-    RASQAL_FREE(intarray, query->variables_use_map);
+    RASQAL_FREE(shortarray, query->variables_use_map);
 
   if(query->query_graph_pattern)
     rasqal_free_graph_pattern(query->query_graph_pattern);
@@ -238,7 +229,7 @@ rasqal_free_query(rasqal_query* query)
     raptor_free_sequence(query->graph_patterns_sequence);
 
   if(query->query_results_formatter_name)
-    RASQAL_FREE(cstring, query->query_results_formatter_name);
+    RASQAL_FREE(char*, query->query_results_formatter_name);
 
   /* Do this last since most everything above could refer to a variable */
   if(query->vars_table)
@@ -253,6 +244,9 @@ rasqal_free_query(rasqal_query* query)
   if(query->bindings)
     rasqal_free_bindings(query->bindings);
   
+  if(query->projection)
+    rasqal_free_projection(query->projection);
+  
   RASQAL_FREE(rasqal_query, query);
 }
 
@@ -433,7 +427,10 @@ rasqal_query_get_distinct(rasqal_query* query)
 {
   RASQAL_ASSERT_OBJECT_POINTER_RETURN_VALUE(query, rasqal_query, 0);
 
-  return query->distinct;
+  if(!query->projection)
+    return 0;
+  
+  return query->projection->distinct;
 }
 
 
@@ -455,10 +452,18 @@ rasqal_query_set_distinct(rasqal_query* query, int distinct_mode)
 {
   RASQAL_ASSERT_OBJECT_POINTER_RETURN(query, rasqal_query);
 
-  if(distinct_mode >= 0 && distinct_mode <= 2)
-    query->distinct= distinct_mode;
-  else
-    query->distinct= 0;
+  if(distinct_mode < 0 || distinct_mode > 2)
+    distinct_mode = 0;
+
+  if(!query->projection) {
+    query->projection = rasqal_new_projection(query,
+                                              /* variables */ NULL,
+                                              /* wildcard */ 0,
+                                              /* distinct */ 0);
+    if(!query->projection)
+      return;
+  }
+  query->projection->distinct = distinct_mode;
 }
 
 
@@ -705,7 +710,7 @@ rasqal_query_dataset_contains_named_graph(rasqal_query* query,
  * @query: #rasqal_query query object
  * @var: #rasqal_variable variable
  *
- * Add a binding variable to the query.
+ * Add a projected (named) variable to the query.
  *
  * See also rasqal_query_set_variable() which assigns or removes a value to
  * a previously added variable in the query.
@@ -718,15 +723,20 @@ rasqal_query_add_variable(rasqal_query* query, rasqal_variable* var)
   RASQAL_ASSERT_OBJECT_POINTER_RETURN_VALUE(query, rasqal_query, 1);
   RASQAL_ASSERT_OBJECT_POINTER_RETURN_VALUE(var, rasqal_variable, 1);
 
-  if(!query->selects) {
-    query->selects = raptor_new_sequence((raptor_data_free_handler)rasqal_free_variable,
-                                         (raptor_data_print_handler)rasqal_variable_print);
-    if(!query->selects)
+  if(!rasqal_variables_table_contains(query->vars_table, var->type, var->name)) {
+    if(rasqal_variables_table_add_variable(query->vars_table, var))
       return 1;
   }
 
-  var = rasqal_new_variable_from_variable(var);
-  return raptor_sequence_push(query->selects, (void*)var);
+  if(!query->projection) {
+    query->projection = rasqal_new_projection(query,
+                                              /* variables */ NULL,
+                                              /* wildcard */ 0,
+                                              /* distinct */ 0);
+    if(!query->projection)
+      return 1;
+  }
+  return rasqal_projection_add_variable(query->projection, var);
 }
 
 
@@ -734,7 +744,7 @@ rasqal_query_add_variable(rasqal_query* query, rasqal_variable* var)
  * rasqal_query_get_bound_variable_sequence:
  * @query: #rasqal_query query object
  *
- * Get the sequence of variables to bind in the query.
+ * Get the sequence of projected variables in the query.
  *
  * This returns the sequence of variables that are explicitly chosen
  * via SELECT in RDQL, SPARQL.  Or all variables mentioned with SELECT *
@@ -746,7 +756,10 @@ rasqal_query_get_bound_variable_sequence(rasqal_query* query)
 {
   RASQAL_ASSERT_OBJECT_POINTER_RETURN_VALUE(query, rasqal_query, NULL);
 
-  return query->selects;
+  if(!query->projection)
+    return NULL;
+  
+  return rasqal_projection_get_variables_sequence(query->projection);
 }
 
 
@@ -809,7 +822,7 @@ rasqal_query_get_all_variable_sequence(rasqal_query* query)
  * @query: #rasqal_query query object
  * @idx: index into the sequence (0 or larger)
  *
- * Get a variable in the sequence of variables to bind.
+ * Get a variable in the query
  *
  * Return value: a #rasqal_variable pointer or NULL if out of range
  **/
@@ -818,19 +831,45 @@ rasqal_query_get_variable(rasqal_query* query, int idx)
 {
   RASQAL_ASSERT_OBJECT_POINTER_RETURN_VALUE(query, rasqal_query, NULL);
 
-  if(!query->selects || idx < 0 || idx >= query->select_variables_count)
-    return NULL;
-  
   return rasqal_variables_table_get(query->vars_table, idx);
 }
 
 
+/**
+ * rasqal_query_has_variable2:
+ * @query: #rasqal_query query object
+ * @type: the variable type to match or #RASQAL_VARIABLE_TYPE_UNKNOWN for any.
+ * @name: variable name
+ *
+ * Find if the named variable of the given type is in the query
+ *
+ * Note that looking up for any type #RASQAL_VARIABLE_TYPE_UNKNOWN
+ * may a name match but for any type so in cases where the query has
+ * both a named and anonymous (extensional) variable, an arbitrary one
+ * will be returned.
+ *
+ * Return value: non-0 if the variable name was found.
+ **/
+int
+rasqal_query_has_variable2(rasqal_query* query,
+                           rasqal_variable_type type,
+                           const unsigned char *name)
+{
+  RASQAL_ASSERT_OBJECT_POINTER_RETURN_VALUE(query, rasqal_query, 0);
+  RASQAL_ASSERT_OBJECT_POINTER_RETURN_VALUE(name, char*, 0);
+
+  return rasqal_variables_table_contains(query->vars_table, type, name);
+}
+
+
 /**
  * rasqal_query_has_variable:
  * @query: #rasqal_query query object
  * @name: variable name
  *
- * Find if the named variable is in the sequence of variables to bind.
+ * Find if the named variable is in the query (of any type)
+ *
+ * @Deprecated: Use rasqal_query_has_variable2() with the variable type arg
  *
  * Return value: non-0 if the variable name was found.
  **/
@@ -840,7 +879,35 @@ rasqal_query_has_variable(rasqal_query* query, const unsigned char *name)
   RASQAL_ASSERT_OBJECT_POINTER_RETURN_VALUE(query, rasqal_query, 0);
   RASQAL_ASSERT_OBJECT_POINTER_RETURN_VALUE(name, char*, 0);
 
-  return rasqal_variables_table_has(query->vars_table, name);
+  return rasqal_query_has_variable2(query, RASQAL_VARIABLE_TYPE_UNKNOWN, name);
+}
+
+
+/**
+ * rasqal_query_set_variable2:
+ * @query: #rasqal_query query object
+ * @type: the variable type to match or #RASQAL_VARIABLE_TYPE_UNKNOWN for any.
+ * @name: #rasqal_variable variable
+ * @value: #rasqal_literal value to set or NULL
+ *
+ * Bind an existing typed variable to a value to the query.
+ *
+ * See also rasqal_query_add_variable() which adds a new binding variable
+ * and must be called before this method is invoked.
+ *
+ * Return value: non-0 on failure
+ **/
+int
+rasqal_query_set_variable2(rasqal_query* query,
+                           rasqal_variable_type type,
+                           const unsigned char *name,
+                           rasqal_literal* value)
+{
+  RASQAL_ASSERT_OBJECT_POINTER_RETURN_VALUE(query, rasqal_query, 1);
+  RASQAL_ASSERT_OBJECT_POINTER_RETURN_VALUE(name, char*, 1);
+  RASQAL_ASSERT_OBJECT_POINTER_RETURN_VALUE(value, rasqal_literal, 1);
+
+  return rasqal_variables_table_set(query->vars_table, type, name, value);
 }
 
 
@@ -850,10 +917,11 @@ rasqal_query_has_variable(rasqal_query* query, const unsigned char *name)
  * @name: #rasqal_variable variable
  * @value: #rasqal_literal value to set or NULL
  *
- * Add a binding variable to the query.
+ * Bind an existing named (selected) variable to a value to the query.
  *
- * See also rasqal_query_add_variable which adds a new binding variable
- * and must be called before this method is invoked.
+ * @Deprecated for rasqal_query_set_variable2() that includes a type
+ * arg.  This function only sets named variables of type
+ * #RASQAL_VARIABLE_TYPE_NORMAL
  *
  * Return value: non-0 on failure
  **/
@@ -861,26 +929,12 @@ int
 rasqal_query_set_variable(rasqal_query* query, const unsigned char *name,
                           rasqal_literal* value)
 {
-  int i;
-
   RASQAL_ASSERT_OBJECT_POINTER_RETURN_VALUE(query, rasqal_query, 1);
   RASQAL_ASSERT_OBJECT_POINTER_RETURN_VALUE(name, char*, 1);
   RASQAL_ASSERT_OBJECT_POINTER_RETURN_VALUE(value, rasqal_literal, 1);
 
-  if(!query->selects)
-    return 1;
-  
-  for(i = 0; i< raptor_sequence_size(query->selects); i++) {
-    rasqal_variable* v;
-    v = (rasqal_variable*)raptor_sequence_get_at(query->selects, i);
-    if(!strcmp((const char*)v->name, (const char*)name)) {
-      if(v->value)
-        rasqal_free_literal(v->value);
-      v->value = value;
-      return 0;
-    }
-  }
-  return 1;
+  return rasqal_query_set_variable2(query, RASQAL_VARIABLE_TYPE_NORMAL,
+                                    name, value);
 }
 
 
@@ -1197,8 +1251,8 @@ rasqal_query_prepare(rasqal_query* query,
      * scanned; thus, scanning consists of `base[0]' through
      * `base[size-2]', inclusive.
      */
-    int len = strlen((const char*)query_string)+3; /* +3 for " \0\0" */
-    unsigned char *query_string_copy = (unsigned char*)RASQAL_MALLOC(cstring, len);
+    size_t len = strlen((const char*)query_string)+3; /* +3 for " \0\0" */
+    unsigned char *query_string_copy = RASQAL_MALLOC(unsigned char*, len);
     if(!query_string_copy) {
       query->failed = 1;
       return 1;
@@ -1268,25 +1322,14 @@ rasqal_query_get_engine_by_name(const char* name)
 {
   const rasqal_query_execution_factory* engine;
 
-#if RASQAL_QUERY_ENGINE_VERSION == 1
-  engine = &rasqal_query_engine_1;
-#else
+  /* default */
   engine = &rasqal_query_engine_algebra;
-#endif
-
-#ifdef RASQAL_DEBUG
-  if(1) {
-    char* n = getenv("RASQAL_DEBUG_ENGINE");
-    if(n)
-      name = n;
-  }
-#endif
 
   if(name) {
-    if(!strcmp(name, "1") || !strcmp(name, "original"))
-      engine = &rasqal_query_engine_1;
-    else if(!strcmp(name, "2") || !strcmp(name, "algebra"))
+    if(!strcmp(name, "2") || !strcmp(name, "algebra"))
       engine = &rasqal_query_engine_algebra;
+    else
+      engine = NULL;
   }
 
   return engine;
@@ -1403,15 +1446,17 @@ rasqal_query_print(rasqal_query* query, FILE *fh)
 {
   rasqal_variables_table* vars_table = query->vars_table;
   raptor_sequence* seq;
+  unsigned int distinct_mode;
 
   RASQAL_ASSERT_OBJECT_POINTER_RETURN_VALUE(query, rasqal_query, 1);
   RASQAL_ASSERT_OBJECT_POINTER_RETURN_VALUE(fh, FILE*, 1);
 
   fprintf(fh, "query verb: %s\n", rasqal_query_verb_as_string(query->verb));
   
-  if(query->distinct)
+  distinct_mode = rasqal_query_get_distinct(query);
+  if(distinct_mode)
     fprintf(fh, "query results distinct mode: %s\n",
-            (query->distinct == 1 ? "distinct" : "reduced"));
+            (distinct_mode == 1 ? "distinct" : "reduced"));
   if(query->explain)
     fputs("query results explain: yes\n", fh);
 
@@ -1435,9 +1480,10 @@ rasqal_query_print(rasqal_query* query, FILE *fh)
     fputs("\nanonymous variables: ", fh); 
     raptor_sequence_print(seq, fh);
   }
-  if(query->selects) {
+  seq = rasqal_query_get_bound_variable_sequence(query);
+  if(seq) {
     fputs("\nbound variables: ", fh); 
-    raptor_sequence_print(query->selects, fh);
+    raptor_sequence_print(seq, fh);
   }
   if(query->describes) {
     fputs("\ndescribes: ", fh);
@@ -1603,7 +1649,35 @@ rasqal_query_get_wildcard(rasqal_query* query)
 {
   RASQAL_ASSERT_OBJECT_POINTER_RETURN_VALUE(query, rasqal_query, 0);
 
-  return query->wildcard;
+  if(!query->projection)
+    return 0;
+  
+  return query->projection->wildcard;
+}
+
+
+/**
+ * rasqal_query_set_wildcard:
+ * @query: #rasqal_query query object
+ * @wildcard: wildcard
+ *
+ * Set the query projection wildcard flag
+ *
+ **/
+void
+rasqal_query_set_wildcard(rasqal_query* query, int wildcard)
+{
+  RASQAL_ASSERT_OBJECT_POINTER_RETURN(query, rasqal_query);
+
+  if(!query->projection) {
+    query->projection = rasqal_new_projection(query,
+                                              /* variables */ NULL,
+                                              /* wildcard */ 0,
+                                              /* wildcard */ 0);
+    if(!query->projection)
+      return;
+  }
+  query->projection->wildcard = wildcard;
 }
 
 
@@ -1876,7 +1950,7 @@ rasqal_query_get_genid(rasqal_query* query, const unsigned char* base,
                        int counter)
 {
   int tmpcounter;
-  int length;
+  size_t length;
   unsigned char *buffer;
 
   RASQAL_ASSERT_OBJECT_POINTER_RETURN_VALUE(query, rasqal_query, NULL);
@@ -1885,12 +1959,12 @@ rasqal_query_get_genid(rasqal_query* query, const unsigned char* base,
   if(counter < 0)
     counter= query->genid_counter++;
   
-  length = strlen((const char*)base)+2;  /* base + (int) + "\0" */
+  length = strlen((const char*)base) + 2;  /* base + (int) + "\0" */
   tmpcounter = counter;
   while(tmpcounter /= 10)
     length++;
   
-  buffer = (unsigned char*)RASQAL_MALLOC(cstring, length);
+  buffer = RASQAL_MALLOC(unsigned char*, length);
   if(!buffer)
     return NULL;
 
@@ -1952,24 +2026,6 @@ rasqal_query_get_variable_by_offset(rasqal_query* query, int idx)
 }
 
 
-int
-rasqal_query_variable_bound_in_triple(rasqal_query *query,
-                                      rasqal_variable *v,
-                                      int column) 
-{
-  int v_column;
-
-  RASQAL_ASSERT_OBJECT_POINTER_RETURN_VALUE(query, rasqal_query, 0);
-  RASQAL_ASSERT_OBJECT_POINTER_RETURN_VALUE(v, rasqal_variable, 0);
-
-  v_column = query->variables_bound_in[v->offset];
-  if(v_column < 0)
-    return 0;
-
-  return v_column == column;
-}
-
-
 /*
  * rasqal_query_add_update_operation:
  * @query: #rasqal_query query object
@@ -2143,19 +2199,29 @@ rasqal_query_get_bindings_row(rasqal_query* query, int idx)
 
 
 /*
- * rasqal_query_get_bound_in_column_for_variable:
+ * rasqal_query_variable_bound_in_triple:
  * @query: #rasqal_query query object
  * @variable: variable
+ * @column: triple column
  * 
- * INTERNAL - Get the column number [triple] that a variable is bound in
+ * INTERNAL - Test if variable is bound in given triple
  *
- * Return value: column or < 0 if not bound
+ * Return value: part of triple the variable is bound in
  */
-int
-rasqal_query_get_bound_in_column_for_variable(rasqal_query* query,
-                                              rasqal_variable* v)
+rasqal_triple_parts
+rasqal_query_variable_bound_in_triple(rasqal_query* query,
+                                      rasqal_variable* v,
+                                      int column)
 {
-  return query->variables_bound_in[v->offset];
+  int width;
+  unsigned short *triple_row;
+  
+  RASQAL_ASSERT_OBJECT_POINTER_RETURN_VALUE(query, rasqal_query, (rasqal_triple_parts)0);
+  
+  width = rasqal_variables_table_get_total_variables_count(query->vars_table);
+  triple_row = &query->triples_use_map[column * width];
+
+  return (rasqal_triple_parts)((triple_row[v->offset] & RASQAL_TRIPLES_BOUND_MASK) >> 4);
 }
 
 
@@ -2208,3 +2274,83 @@ rasqal_query_get_result_type(rasqal_query* query)
 
   return type;
 }
+
+
+/*
+ * rasqal_query_store_select_query:
+ * @query: query
+ * @gp: SELECT graph pattern
+ *
+ * INTERNAL - store a select query
+ *
+ * The query object owns the @projection, @data_graphs, @where_gp and
+ * the @modifier after this call.
+ *
+ * Return value: non-0 on failure
+ */
+int
+rasqal_query_store_select_query(rasqal_query* query,
+                                rasqal_projection* projection,
+                                raptor_sequence* data_graphs,
+                                rasqal_graph_pattern* where_gp,
+                                rasqal_solution_modifier* modifier)
+{
+  if(!projection || !where_gp || !modifier)
+    return 1;
+
+  query->verb = RASQAL_QUERY_VERB_SELECT;
+
+  rasqal_query_set_projection(query, projection);
+
+  query->query_graph_pattern = where_gp;
+  
+  if(data_graphs)
+    rasqal_query_add_data_graphs(query, data_graphs);
+
+  query->modifier = modifier;
+
+  return 0;
+}
+
+
+/*
+ * rasqal_query_get_projection:
+ * @query: #rasqal_query
+ *
+ * INTERNAL - Get the query variable projection
+ *
+ * This may be NULL if the query does not project any variables such
+ * as for CONSTRUCT or ASK queries.
+ *
+ * Return value: projection or NULL.
+ **/
+rasqal_projection*
+rasqal_query_get_projection(rasqal_query* query)
+{
+  RASQAL_ASSERT_OBJECT_POINTER_RETURN_VALUE(query, rasqal_query, NULL);
+
+  return query->projection;
+}
+
+
+/*
+ * rasqal_query_set_projection:
+ * @query: #rasqal_query
+ * @projection: variable projection and flags
+ *
+ * INTERNAL - Set the query projection
+ *
+ * Return value: non-0 on failure
+ **/
+int
+rasqal_query_set_projection(rasqal_query* query, rasqal_projection* projection)
+{
+  RASQAL_ASSERT_OBJECT_POINTER_RETURN_VALUE(query, rasqal_query, 1);
+
+  if(query->projection)
+    rasqal_free_projection(query->projection);
+
+  query->projection = projection;
+  
+  return 0;
+}
