@@ -49,7 +49,7 @@
 #define DEBUG_FH stderr
 
 /* prototype for later */
-static int rasqal_query_build_variables_use_map(rasqal_query* query);
+static int rasqal_query_build_variables_use_map(rasqal_query* query, rasqal_projection* projection);
 static int rasqal_query_graph_build_variables_use_map_binds(rasqal_graph_pattern* gp, unsigned short* vars_scope);
 static void rasqal_query_expression_build_variables_use_map(unsigned short *use_map, rasqal_expression* e);
 static void rasqal_query_let_build_variables_use_map(rasqal_query* query, unsigned short *use_map, rasqal_expression* e);
@@ -250,31 +250,23 @@ rasqal_query_build_anonymous_variables(rasqal_query* rq)
  * Return value: non-0 on failure
  */
 int
-rasqal_query_expand_wildcards(rasqal_query* rq)
+rasqal_query_expand_wildcards(rasqal_query* rq, rasqal_projection* projection)
 {
   int i;
   int size;
-  
-  if(rq->verb != RASQAL_QUERY_VERB_SELECT || !rq->wildcard)
+
+  if(rq->verb != RASQAL_QUERY_VERB_SELECT || 
+     !projection || !projection->wildcard)
     return 0;
   
   /* If 'SELECT *' was given, make the selects be a list of all variables */
-  rq->selects = raptor_new_sequence((raptor_data_free_handler)rasqal_free_variable,
-                                    (raptor_data_print_handler)rasqal_variable_print);
-  if(!rq->selects)
-    return 1;
-  
   size = rasqal_variables_table_get_named_variables_count(rq->vars_table);
   for(i = 0; i < size; i++) {
     rasqal_variable* v = rasqal_variables_table_get(rq->vars_table, i);
 
-    v = rasqal_new_variable_from_variable(v);
-    if(raptor_sequence_push(rq->selects, v))
-      return 1;
+    rasqal_query_add_variable(rq, v);
   }
 
-  rq->select_variables_count = size;
-
   return 0;
 }
 
@@ -290,17 +282,22 @@ rasqal_query_expand_wildcards(rasqal_query* rq)
  * Return value: non-0 on failure
  */
 int
-rasqal_query_remove_duplicate_select_vars(rasqal_query* rq)
+rasqal_query_remove_duplicate_select_vars(rasqal_query* rq,
+                                          rasqal_projection* projection)
 {
   int i;
   int modified = 0;
   int size;
-  raptor_sequence* seq = rq->selects;
+  raptor_sequence* seq;
   raptor_sequence* new_seq;
-  
-  if(!seq)
+
+  if(!projection)
     return 1;
 
+  seq = projection->variables;
+  if(!seq)
+    return 0;
+
   size = raptor_sequence_size(seq);
   if(!size)
     return 0;
@@ -312,7 +309,7 @@ rasqal_query_remove_duplicate_select_vars(rasqal_query* rq)
   
 #if RASQAL_DEBUG > 1
   RASQAL_DEBUG1("bound variables before deduping: "); 
-  raptor_sequence_print(rq->selects, DEBUG_FH);
+  raptor_sequence_print(seq, DEBUG_FH);
   fputs("\n", DEBUG_FH); 
 #endif
 
@@ -353,9 +350,8 @@ rasqal_query_remove_duplicate_select_vars(rasqal_query* rq)
     raptor_sequence_print(new_seq, DEBUG_FH);
     fputs("\n", DEBUG_FH); 
 #endif
-    raptor_free_sequence(rq->selects);
-    rq->selects = new_seq;
-    rq->select_variables_count = raptor_sequence_size(rq->selects);
+    raptor_free_sequence(projection->variables);
+    projection->variables = new_seq;
   } else
     raptor_free_sequence(new_seq);
 
@@ -382,8 +378,7 @@ rasqal_query_build_variable_agg_use(rasqal_query* query)
   width = rasqal_variables_table_get_total_variables_count(query->vars_table);
   height = RASQAL_VAR_USE_MAP_OFFSET_LAST + 1 + query->graph_pattern_count;
 
-  agg_row = (unsigned short*)RASQAL_CALLOC(intarray, width,
-                                           sizeof(unsigned short));
+  agg_row = RASQAL_CALLOC(unsigned short*, width, sizeof(unsigned short));
   if(!agg_row)
     return NULL;
 
@@ -1054,14 +1049,15 @@ rasqal_query_prepare_count_graph_patterns(rasqal_query* query,
  * Return value: non-0 on failure
  */
 int
-rasqal_query_build_variables_use(rasqal_query* query)
+rasqal_query_build_variables_use(rasqal_query* query,
+                                 rasqal_projection* projection)
 {
   int rc;
   
   /* create query->variables_use_map that marks where a variable is 
    * mentioned, bound or used in a graph pattern.
    */
-  rc = rasqal_query_build_variables_use_map(query); 
+  rc = rasqal_query_build_variables_use_map(query, projection);
   if(rc)
     return rc;
   
@@ -1136,13 +1132,15 @@ int
 rasqal_query_prepare_common(rasqal_query *query)
 {
   int rc = 1;
+  rasqal_projection* projection;
 
   if(!query->triples)
     goto done;
   
-  /* turn SELECT $a, $a into SELECT $a - editing query->selects */
-  if(query->selects) {
-    if(rasqal_query_remove_duplicate_select_vars(query))
+  /* turn SELECT $a, $a into SELECT $a - editing the projection */
+  projection = rasqal_query_get_projection(query);
+  if(projection) {
+    if(rasqal_query_remove_duplicate_select_vars(query, projection))
       goto done;
   }
 
@@ -1219,7 +1217,7 @@ rasqal_query_prepare_common(rasqal_query *query)
                                      rasqal_query_prepare_count_graph_patterns,
                                      query->graph_patterns_sequence);
 
-    rc = rasqal_query_build_variables_use(query);
+    rc = rasqal_query_build_variables_use(query, projection);
     if(rc)
       goto done;
     
@@ -1327,6 +1325,8 @@ rasqal_graph_patterns_join(rasqal_graph_pattern *dest_gp,
     src_gp->modifier = NULL;
   }
 
+  dest_gp->silent = src_gp->silent;
+
 #if RASQAL_DEBUG > 1
   RASQAL_DEBUG2("Result graph pattern #%d\n  ", dest_gp->gp_index);
   rasqal_graph_pattern_print(dest_gp, stdout);
@@ -1702,6 +1702,25 @@ rasqal_query_triples_build_variables_use_map_binds(rasqal_query* query,
 }
 
 
+#ifdef RASQAL_DEBUG
+static void
+rasqal_query_dump_vars_scope(rasqal_query* query, int width, unsigned short *vars_scope)
+{
+  int j;
+  
+  for(j = 0; j < width; j++) {
+    rasqal_variable* v = rasqal_variables_table_get(query->vars_table, j);
+    fprintf(DEBUG_FH, "%8s ", v->name);
+  }
+  fputs("\n  ", DEBUG_FH);
+  for(j = 0; j < width; j++) {
+    fprintf(DEBUG_FH, "%8d ", vars_scope[j]);
+  }
+  fputc('\n', DEBUG_FH);
+}
+#endif
+
+
 /**
  * rasqal_query_graph_pattern_build_variables_use_map_binds:
  * @use_map: 2D array of (num. variables x num. GPs) to READ and WRITE
@@ -1754,23 +1773,28 @@ rasqal_query_graph_pattern_build_variables_use_map_binds(rasqal_query* query,
       break;
       
     case RASQAL_GRAPH_PATTERN_OPERATOR_UNION:
+    case RASQAL_GRAPH_PATTERN_OPERATOR_GROUP:
+    case RASQAL_GRAPH_PATTERN_OPERATOR_OPTIONAL:
       rc = rasqal_query_union_build_variables_use_map_binds(query,
                                                             use_map,
                                                             width,
                                                             gp, vars_scope);
       break;
       
-    case RASQAL_GRAPH_PATTERN_OPERATOR_OPTIONAL:
-    case RASQAL_GRAPH_PATTERN_OPERATOR_GROUP:
     case RASQAL_GRAPH_PATTERN_OPERATOR_SERVICE:
     case RASQAL_GRAPH_PATTERN_OPERATOR_MINUS:
     case RASQAL_GRAPH_PATTERN_OPERATOR_UNKNOWN:
       break;
   }
   
+#ifdef RASQAL_DEBUG
+  RASQAL_DEBUG3("vars_scope after %s graph pattern #%d verb is now:\n  ",
+                rasqal_graph_pattern_operator_as_string(gp->op), gp->gp_index);
+  rasqal_query_dump_vars_scope(query, width, vars_scope);
+#endif
 
-  /* Bind sub-graph patterns */
-  if(gp->graph_patterns) {
+  /* Bind sub-graph patterns but not sub-SELECT gp twice */
+  if(gp->op != RASQAL_GRAPH_PATTERN_OPERATOR_SELECT && gp->graph_patterns) {
     int gp_size = raptor_sequence_size(gp->graph_patterns);
     int i;
     
@@ -1779,12 +1803,19 @@ rasqal_query_graph_pattern_build_variables_use_map_binds(rasqal_query* query,
       rasqal_graph_pattern *sgp;
       
       sgp = (rasqal_graph_pattern*)raptor_sequence_get_at(gp->graph_patterns, i);
+      RASQAL_DEBUG2("checking vars_scope for SGP #%d\n", sgp->gp_index);
+
       rc = rasqal_query_graph_pattern_build_variables_use_map_binds(query,
                                                                     use_map,
                                                                     width, sgp,
                                                                     vars_scope);
       if(rc)
         goto done;
+
+#ifdef RASQAL_DEBUG
+      RASQAL_DEBUG2("vars_scope after SGP #%d is now:\n  ", sgp->gp_index);
+      rasqal_query_dump_vars_scope(query, width, vars_scope);
+#endif
     }
   }
 
@@ -1799,28 +1830,29 @@ rasqal_query_graph_pattern_build_variables_use_map_binds(rasqal_query* query,
  * @query: the #rasqal_query to find the variables in
  * @use_map: 2D array of (num. variables x num. GPs) to READ and WRITE
  * @width: width of array (num. variables)
+ * @gp: graph pattern to use
  *
- * INTERNAL - Calculate which GPs bind variables in a graph_pattern query
+ * INTERNAL - Calculate which GPs bind variables in a graph_pattern tree walk
  * 
  * Return value: non-0 on failure
  **/
 static int
 rasqal_query_build_variables_use_map_binds(rasqal_query* query,
                                            unsigned short *use_map,
-                                           int width)
+                                           int width,
+                                           rasqal_graph_pattern *gp)
 {
   int rc;
   unsigned short* vars_scope;
 
-  vars_scope = (unsigned short*)RASQAL_CALLOC(intarray,  width,
-                                              sizeof(unsigned short));
+  vars_scope = RASQAL_CALLOC(unsigned short*, width, sizeof(unsigned short));
   if(!vars_scope)
     return 1;
   
   rc = rasqal_query_graph_pattern_build_variables_use_map_binds(query,
                                                                 use_map,
                                                                 width,
-                                                                query->query_graph_pattern,
+                                                                gp,
                                                                 vars_scope);
   RASQAL_FREE(intarray, vars_scope);
 
@@ -1929,7 +1961,7 @@ rasqal_query_print_triples_use_map(FILE* fh, rasqal_query* query)
     fprintf(fh, "%-7d ", column);
     for(i = 0; i < width; i++) {
       int flag = row[i];
-      char label[13]="            \0";
+      char label[14] = "             ";
       int bit;
       
       /* bit 0: RASQAL_TRIPLES_USE_SUBJECT
@@ -2083,7 +2115,8 @@ rasqal_query_build_expressions_sequence_use_map_row(unsigned short* use_map_row,
  * Return value: non-0 on failure
  **/
 static int
-rasqal_query_build_variables_use_map(rasqal_query* query)
+rasqal_query_build_variables_use_map(rasqal_query* query,
+                                     rasqal_projection* projection)
 {
   int width;
   int height;
@@ -2095,8 +2128,7 @@ rasqal_query_build_variables_use_map(rasqal_query* query)
   width = rasqal_variables_table_get_total_variables_count(query->vars_table);
   height = RASQAL_VAR_USE_MAP_OFFSET_LAST + 1 + query->graph_pattern_count;
   
-  use_map = (unsigned short*)RASQAL_CALLOC(shortarray,  width * height,
-                                           sizeof(unsigned short));
+  use_map = RASQAL_CALLOC(unsigned short*, width * height, sizeof(unsigned short));
   if(!use_map)
     return 1;
 
@@ -2106,8 +2138,7 @@ rasqal_query_build_variables_use_map(rasqal_query* query)
   query->variables_use_map = use_map;
 
   height = raptor_sequence_size(query->triples);
-  use_map = (unsigned short*)RASQAL_CALLOC(shortarray, width * height,
-                                           sizeof(unsigned short));
+  use_map = RASQAL_CALLOC(unsigned short*, width * height, sizeof(unsigned short));
   if(!use_map) {
     RASQAL_FREE(shortarray, query->variables_use_map);
     query->variables_use_map = NULL;
@@ -2126,8 +2157,9 @@ rasqal_query_build_variables_use_map(rasqal_query* query)
   switch(query->verb) {
     case RASQAL_QUERY_VERB_SELECT:
       /* This also handles 1a) select/project expressions */
-      rc = rasqal_query_build_variables_sequence_use_map_row(use_map_row,
-                                                             query->selects);
+      if(projection && projection->variables)
+        rc = rasqal_query_build_variables_sequence_use_map_row(use_map_row,
+                                                               projection->variables);
       break;
   
     case RASQAL_QUERY_VERB_DESCRIBE:
@@ -2202,24 +2234,25 @@ rasqal_query_build_variables_use_map(rasqal_query* query)
   
 #ifdef RASQAL_DEBUG
   RASQAL_DEBUG1("variables use map after mentions: ");
-  rasqal_query_print_variables_use_map(stderr, query);
+  rasqal_query_print_variables_use_map(DEBUG_FH, query);
   fputs("\n", DEBUG_FH); 
 #endif    
 
   /* calculate which GPs bind variables for all query graph patterns
    * reading from the use_map
    */
-  rc = rasqal_query_build_variables_use_map_binds(query, use_map, width);
+  rc = rasqal_query_build_variables_use_map_binds(query, use_map, width,
+                                                  query->query_graph_pattern);
   if(rc)
     goto done;
 
 #ifdef RASQAL_DEBUG
   RASQAL_DEBUG1("use map after binds and mentions: ");
-  rasqal_query_print_variables_use_map(stderr, query);
+  rasqal_query_print_variables_use_map(DEBUG_FH, query);
   fputs("\n", DEBUG_FH); 
 
   RASQAL_DEBUG1("triples use map after binds and mentions: ");
-  rasqal_query_print_triples_use_map(stderr, query);
+  rasqal_query_print_triples_use_map(DEBUG_FH, query);
   fputs("\n", DEBUG_FH); 
 #endif    
 
@@ -2399,8 +2432,7 @@ rasqal_query_select_build_variables_use_map_binds(rasqal_query* query,
   int size;
   int i;
 
-  inner_vars_scope = (unsigned short*)RASQAL_CALLOC(intarray,  width, 
-                                                    sizeof(unsigned short));
+  inner_vars_scope = RASQAL_CALLOC(unsigned short*, width, sizeof(unsigned short));
   if(!inner_vars_scope)
     return 1;
 
@@ -2463,8 +2495,7 @@ rasqal_query_union_build_variables_use_map_binds(rasqal_query* query,
   seq = gp->graph_patterns;
   gp_size = raptor_sequence_size(seq);
   
-  inner_vars_scope = (unsigned short*)RASQAL_CALLOC(intarray,  width,
-                                                    sizeof(unsigned short));
+  inner_vars_scope = RASQAL_CALLOC(unsigned short*, width, sizeof(unsigned short));
   if(!inner_vars_scope)
     return 1;
 
