@@ -160,10 +160,10 @@ rasqal_query_write_sparql_literal(sparql_writer_context *wc,
         raptor_iostream_write_byte('"', iostr);
         raptor_string_ntriples_write(l->string, l->string_len, '"', iostr);
         raptor_iostream_counted_string_write("\"^^", 3, iostr);
-        if(l->type == RASQAL_LITERAL_UDT) 
-          dt_uri = l->datatype;
-        else
+        if(l->type <= RASQAL_LITERAL_LAST_XSD)
           dt_uri = rasqal_xsd_datatype_type_to_uri(l->world, l->type);
+        else
+          dt_uri = l->datatype;
         rasqal_query_write_sparql_uri(wc, iostr, dt_uri);
       }
       break;
@@ -290,7 +290,16 @@ static const char* const rasqal_sparql_op_labels[RASQAL_EXPR_LAST+1] = {
   "ABS",
   "ROUND",
   "CEIL",
-  "FLOOR"
+  "FLOOR",
+  "MD5",
+  "SHA1",
+  "SHA224",
+  "SHA256",
+  "SHA384",
+  "SHA512",
+  "STRBEFORE",
+  "STRAFTER",
+  "REPLACE"
 };
 
 
@@ -349,6 +358,8 @@ rasqal_query_write_sparql_expression(sparql_writer_context *wc,
     case RASQAL_EXPR_STRSTARTS:
     case RASQAL_EXPR_STRENDS:
     case RASQAL_EXPR_CONTAINS:
+    case RASQAL_EXPR_STRBEFORE:
+    case RASQAL_EXPR_STRAFTER:
       raptor_iostream_counted_string_write("( ", 2, iostr);
       rasqal_query_write_sparql_expression(wc, iostr, e->arg1);
       raptor_iostream_write_byte(' ', iostr);
@@ -398,6 +409,12 @@ rasqal_query_write_sparql_expression(sparql_writer_context *wc,
     case RASQAL_EXPR_ROUND:
     case RASQAL_EXPR_CEIL:
     case RASQAL_EXPR_FLOOR:
+    case RASQAL_EXPR_MD5:
+    case RASQAL_EXPR_SHA1:
+    case RASQAL_EXPR_SHA224:
+    case RASQAL_EXPR_SHA256:
+    case RASQAL_EXPR_SHA384:
+    case RASQAL_EXPR_SHA512:
       rasqal_query_write_sparql_expression_op(wc, iostr, e);
       raptor_iostream_counted_string_write("( ", 2, iostr);
       rasqal_query_write_sparql_expression(wc, iostr, e->arg1);
@@ -408,6 +425,7 @@ rasqal_query_write_sparql_expression(sparql_writer_context *wc,
     case RASQAL_EXPR_REGEX:
     case RASQAL_EXPR_IF:
     case RASQAL_EXPR_SUBSTR:
+    case RASQAL_EXPR_REPLACE:
       rasqal_query_write_sparql_expression_op(wc, iostr, e);
       raptor_iostream_counted_string_write("( ", 2, iostr);
       rasqal_query_write_sparql_expression(wc, iostr, e->arg1);
@@ -537,7 +555,7 @@ rasqal_query_write_sparql_triple_data(sparql_writer_context *wc,
   
   /* look for triples */
   while(1) {
-    rasqal_triple* t = raptor_sequence_get_at(triples, triple_index);
+    rasqal_triple* t = (rasqal_triple*)raptor_sequence_get_at(triples, triple_index);
     if(!t)
       break;
     
@@ -801,7 +819,7 @@ rasqal_write_sparql_bindings(sparql_writer_context* wc,
   
     for(i = 0; i < raptor_sequence_size(bindings->rows); i++) {
       rasqal_row* row;
-      row = raptor_sequence_get_at(bindings->rows, i);
+      row = (rasqal_row*)raptor_sequence_get_at(bindings->rows, i);
       raptor_iostream_counted_string_write("  ", 2, iostr);
       rasqal_write_sparql_row(wc, iostr, row);
       raptor_iostream_counted_string_write("\n", 1, iostr);
@@ -849,6 +867,29 @@ rasqal_query_write_graphref(sparql_writer_context* wc,
 }
 
 
+static int
+rasqal_query_write_sparql_projection(sparql_writer_context *wc,
+                                     raptor_iostream *iostr, 
+                                     rasqal_projection* projection)
+{
+  if(!projection)
+    return 1;
+
+  if(projection->distinct) {
+    if(projection->distinct == 1)
+      raptor_iostream_counted_string_write(" DISTINCT", 9, iostr);
+    else
+      raptor_iostream_counted_string_write(" REDUCED", 8, iostr);
+  }
+
+  if(projection->wildcard) {
+    raptor_iostream_counted_string_write(" *", 2, iostr);
+    return 0;
+  }
+
+  return rasqal_query_write_sparql_select(wc, iostr, projection->variables);
+}
+
 
 int
 rasqal_query_write_sparql_20060406(raptor_iostream *iostr, 
@@ -858,7 +899,8 @@ rasqal_query_write_sparql_20060406(raptor_iostream *iostr,
   sparql_writer_context wc;
   int limit, offset;
   rasqal_query_verb verb;
-
+  rasqal_projection* projection;
+  
   wc.world = query->world;
   wc.base_uri = NULL;
 
@@ -1014,16 +1056,11 @@ rasqal_query_write_sparql_20060406(raptor_iostream *iostr,
     raptor_iostream_string_write(rasqal_query_verb_as_string(query->verb),
                                  iostr);
 
-  if(query->distinct) {
-    if(query->distinct == 1)
-      raptor_iostream_counted_string_write(" DISTINCT", 9, iostr);
-    else
-      raptor_iostream_counted_string_write(" REDUCED", 8, iostr);
-  }
-
-  if(query->wildcard)
-    raptor_iostream_counted_string_write(" *", 2, iostr);
-  else if(verb == RASQAL_QUERY_VERB_DESCRIBE) {
+  projection = rasqal_query_get_projection(query);
+  if(projection)
+    rasqal_query_write_sparql_projection(&wc, iostr, projection);
+  
+  if(verb == RASQAL_QUERY_VERB_DESCRIBE) {
     raptor_sequence *lit_seq = query->describes;
     int count = raptor_sequence_size(lit_seq);
 
@@ -1032,9 +1069,8 @@ rasqal_query_write_sparql_20060406(raptor_iostream *iostr,
       raptor_iostream_write_byte(' ', iostr);
       rasqal_query_write_sparql_literal(&wc, iostr, l);
     }
-  } else if(verb == RASQAL_QUERY_VERB_SELECT) {
-    rasqal_query_write_sparql_select(&wc, iostr, query->selects);
   }
+
   raptor_iostream_write_byte('\n', iostr);
 
   if(query->data_graphs) {
@@ -1135,13 +1171,13 @@ rasqal_query_write_sparql_20060406(raptor_iostream *iostr,
   offset = rasqal_query_get_offset(query);
   if(limit >= 0 || offset >= 0) {
     if(limit >= 0) {
-      raptor_iostream_counted_string_write("LIMIT ", 7, iostr);
+      raptor_iostream_counted_string_write("LIMIT ", 6, iostr);
       raptor_iostream_decimal_write(limit, iostr);
     }
     if(offset >= 0) {
       if(limit)
         raptor_iostream_write_byte(' ', iostr);
-      raptor_iostream_counted_string_write("OFFSET ", 8, iostr);
+      raptor_iostream_counted_string_write("OFFSET ", 7, iostr);
       raptor_iostream_decimal_write(offset, iostr);
     }
     raptor_iostream_write_byte('\n', iostr);
