@@ -90,7 +90,7 @@ rasqal_world_register_query_results_format_factory(rasqal_world* world,
 
 #if defined(RASQAL_DEBUG) && RASQAL_DEBUG > 1
   RASQAL_DEBUG3("Registered query result format %s with context size %d\n",
-                factory->names[0], factory->context_length);
+                factory->desc.names[0], factory->context_length);
 #endif
 
   return factory;
@@ -574,12 +574,12 @@ rasqal_world_guess_query_results_format_name(rasqal_world* world,
   rasqal_query_results_format_factory *factory;
   unsigned char *suffix = NULL;
   struct syntax_score* scores;
-
+  int size;
+  
   RASQAL_ASSERT_OBJECT_POINTER_RETURN_VALUE(world, rasqal_world, NULL);
 
-  scores = RASQAL_CALLOC(struct syntax_score*, 
-                         raptor_sequence_size(world->query_results_formats),
-                         sizeof(struct syntax_score));
+  size = raptor_sequence_size(world->query_results_formats);
+  scores = RASQAL_CALLOC(struct syntax_score*, size, sizeof(struct syntax_score));
   if(!scores)
     return NULL;
   
@@ -669,7 +669,7 @@ rasqal_world_guess_query_results_format_name(rasqal_world* world,
     scores[i].score = score < 10 ? score : 10; 
     scores[i].factory = factory;
 #if RASQAL_DEBUG > 2
-    RASQAL_DEBUG3("Score %15s : %d\n", factory->name, score);
+    RASQAL_DEBUG3("Score %15s : %d\n", factory->desc.names[0], score);
 #endif
   }
   
