@@ -45,9 +45,12 @@
 
 #ifndef STANDALONE
 
-#define INIT_RIGHT  (0)
-#define READ_RIGHT (1)
-#define FINISHED   (2)
+typedef enum {
+  JS_START,
+  JS_INIT_RIGHT,
+  JS_READ_RIGHT,
+  JS_FINISHED,
+} rasqal_join_state;
 
 typedef struct 
 {
@@ -62,7 +65,7 @@ typedef struct
   int* right_map;
 
   /* 0 = reading from left rs, 1 = reading from right rs, 2 = finished */
-  int state;
+  rasqal_join_state state;
 
   int failed;
 
@@ -92,15 +95,9 @@ rasqal_join_rowsource_init(rasqal_rowsource* rowsource, void *user_data)
   con = (rasqal_join_rowsource_context*)user_data;
 
   con->failed = 0;
+  con->state = JS_START;
 
-  con->left_row  = rasqal_rowsource_read_row(con->left);
-  if(!con->left_row) {
-    con->state = FINISHED;
-    return 0;
-  }
-
-  con->state = INIT_RIGHT;
-
+  /* If join condition is a constant - optimize it away */
   if(con->expr && rasqal_expression_is_constant(con->expr)) {
     rasqal_query *query = rowsource->query;
     rasqal_literal* result;
@@ -138,7 +135,7 @@ rasqal_join_rowsource_init(rasqal_rowsource* rowsource, void *user_data)
 
     if(!bresult) {
       /* Constraint is always false so row source is finished */
-      con->state = 2;
+      con->state = JS_FINISHED;
     }
     /* otherwise always true so no need to evaluate on each row
      * and deleting con->expr will handle that
@@ -146,8 +143,8 @@ rasqal_join_rowsource_init(rasqal_rowsource* rowsource, void *user_data)
 
   }
 
-  rasqal_rowsource_set_preserve(con->left, 1);
-  rasqal_rowsource_set_preserve(con->right, 1);
+  rasqal_rowsource_set_requirements(con->left, RASQAL_ROWSOURCE_REQUIRE_RESET);
+  rasqal_rowsource_set_requirements(con->right, RASQAL_ROWSOURCE_REQUIRE_RESET);
   
   vars_table = con->left->vars_table;
   con->rc_map = rasqal_new_row_compatible(vars_table, con->left, con->right);
@@ -210,7 +207,7 @@ rasqal_join_rowsource_ensure_variables(rasqal_rowsource* rowsource,
     return 1;
 
   map_size = rasqal_rowsource_get_size(con->right);
-  con->right_map = (int*)RASQAL_MALLOC(int, sizeof(int) * map_size);
+  con->right_map = RASQAL_MALLOC(int*, sizeof(int) * map_size);
   if(!con->right_map)
     return 1;
 
@@ -306,7 +303,7 @@ rasqal_join_rowsource_read_row(rasqal_rowsource* rowsource, void *user_data)
 
   con = (rasqal_join_rowsource_context*)user_data;
 
-  if(con->failed || con->state == FINISHED)
+  if(con->failed || con->state == JS_FINISHED)
     return NULL;
 
   while(1) {
@@ -314,9 +311,28 @@ rasqal_join_rowsource_read_row(rasqal_rowsource* rowsource, void *user_data)
     int bresult = 1;
     int compatible = 1;
 
-    if(con->state == INIT_RIGHT) {
+    if(con->state == JS_START) {
+      /* start / re-start left */
+      if(con->left_row)
+        rasqal_free_row(con->left_row);
+
+      con->left_row  = rasqal_rowsource_read_row(con->left);
+#ifdef RASQAL_DEBUG
+      RASQAL_DEBUG2("rowsource %p read left row : ", rowsource);
+      if(con->left_row)
+        rasqal_row_print(con->left_row, stderr);
+      else
+        fputs("NONE", stderr);
+      fputs("\n", stderr);
+#endif
+      con->state = JS_INIT_RIGHT;
+    }
+
+    if(con->state == JS_INIT_RIGHT) {
+      /* start right */
+
       if(!con->left_row) {
-        con->state = FINISHED;
+        con->state = JS_FINISHED;
         return NULL;
       }
 
@@ -327,8 +343,20 @@ rasqal_join_rowsource_read_row(rasqal_rowsource* rowsource, void *user_data)
 
 
     right_row = rasqal_rowsource_read_row(con->right);
-    if(!right_row && con->state == READ_RIGHT) {
-      /* right table done */
+#ifdef RASQAL_DEBUG
+    RASQAL_DEBUG2("rowsource %p read right row : ", rowsource);
+    if(right_row)
+      rasqal_row_print(right_row, stderr);
+    else
+      fputs("NONE", stderr);
+    fputs("\n", stderr);
+#endif
+
+    if(!right_row && con->state == JS_READ_RIGHT) {
+      /* right has finished */
+
+      /* restart left */
+      con->state = JS_START;
 
       /* if all right table returned no bindings, return left row */
       if(!con->right_rows_joined_count) {
@@ -344,20 +372,13 @@ rasqal_join_rowsource_read_row(rasqal_rowsource* rowsource, void *user_data)
         }
       }
 
-      if(right_row)
-        rasqal_free_row(right_row);
-
-      /* restart left, continue looping */
-      con->state = INIT_RIGHT;
-      if(con->left_row)
-        rasqal_free_row(con->left_row);
-      con->left_row = rasqal_rowsource_read_row(con->left);
+      /* restart left by continuing the loop */
       continue;
     }
 
 
-    /* state is always READ_RIGHT at this point */
-    con->state = READ_RIGHT;
+    /* state is always JS_READ_RIGHT at this point */
+    con->state = JS_READ_RIGHT;
 
     /* now may have both left and right rows so compute compatibility */
     if(right_row) {
@@ -422,7 +443,8 @@ rasqal_join_rowsource_read_row(rasqal_rowsource* rowsource, void *user_data)
         row = rasqal_join_rowsource_build_merged_row(rowsource, con, right_row);
         break;
       }
-    
+
+#if 0    
       /*
        * { mu1 | mu1 in Omega1 and mu2 in Omega2, and mu1 and mu2 are
        * not compatible }
@@ -441,6 +463,7 @@ rasqal_join_rowsource_read_row(rasqal_rowsource* rowsource, void *user_data)
           }
         }
       }
+#endif
 
       /*
        * { mu1 | mu1 in Omega1 and mu2 in Omega2, and mu1 and mu2 are
@@ -462,6 +485,8 @@ rasqal_join_rowsource_read_row(rasqal_rowsource* rowsource, void *user_data)
   if(row) {
     row->rowsource = rowsource;
     row->offset = con->offset++;
+
+    rasqal_row_bind_variables(row, rowsource->query->vars_table);
   }
   
   return row;
@@ -476,7 +501,7 @@ rasqal_join_rowsource_reset(rasqal_rowsource* rowsource, void *user_data)
   
   con = (rasqal_join_rowsource_context*)user_data;
 
-  con->state = INIT_RIGHT;
+  con->state = JS_START;
   con->failed = 0;
   
   rc = rasqal_rowsource_reset(con->left);
@@ -487,23 +512,6 @@ rasqal_join_rowsource_reset(rasqal_rowsource* rowsource, void *user_data)
 }
 
 
-static int
-rasqal_join_rowsource_set_preserve(rasqal_rowsource* rowsource,
-                                   void *user_data, int preserve)
-{
-  rasqal_join_rowsource_context *con;
-  int rc;
-
-  con = (rasqal_join_rowsource_context*)user_data;
-
-  rc = rasqal_rowsource_set_preserve(con->left, preserve);
-  if(rc)
-    return rc;
-
-  return rasqal_rowsource_set_preserve(con->right, preserve);
-}
-
-
 static rasqal_rowsource*
 rasqal_join_rowsource_get_inner_rowsource(rasqal_rowsource* rowsource,
                                           void *user_data, int offset)
@@ -529,7 +537,7 @@ static const rasqal_rowsource_handler rasqal_join_rowsource_handler = {
   /* .read_row = */ rasqal_join_rowsource_read_row,
   /* .read_all_rows = */ NULL,
   /* .reset = */ rasqal_join_rowsource_reset,
-  /* .set_preserve = */ rasqal_join_rowsource_set_preserve,
+  /* .set_requirements = */ NULL,
   /* .get_inner_rowsource = */ rasqal_join_rowsource_get_inner_rowsource,
   /* .set_origin = */ NULL,
 };
@@ -574,7 +582,7 @@ rasqal_new_join_rowsource(rasqal_world *world,
      join_type != RASQAL_JOIN_TYPE_NATURAL)
     goto fail;
   
-  con = (rasqal_join_rowsource_context*)RASQAL_CALLOC(rasqal_join_rowsource_context, 1, sizeof(rasqal_join_rowsource_context));
+  con = RASQAL_CALLOC(rasqal_join_rowsource_context*, 1, sizeof(*con));
   if(!con)
     goto fail;
 
@@ -623,7 +631,9 @@ const char* const join_1_data_2x3_rows[] =
 };
   
 
-const char* const join_2_data_3x4_rows[] =
+/* join on b */
+
+const char* const join_2_data_3x2_rows[] =
 {
   /* 3 variable names and 4 rows */
   "b",     NULL, "c",      NULL, "d",      NULL,
@@ -631,16 +641,11 @@ const char* const join_2_data_3x4_rows[] =
   "red",   NULL, "orange", NULL, "yellow", NULL,
   /* row 2 data */
   "blue",  NULL, "indigo", NULL, "violet", NULL,
-  /* row 3 data */
-  "blue", NULL, "silver", NULL, "gold",   NULL,
-  /* row 4 data */
-  "green", NULL, "tope",   NULL, "bronze", NULL,
   /* end of data */
   NULL, NULL, NULL, NULL, NULL, NULL
 };
 
 
-/* join on b=red: 1 row b=blue: 2 rows and b=green: 1 row */
 typedef struct {
   rasqal_join_type join_type;
   int expected;
@@ -648,8 +653,8 @@ typedef struct {
 
 #define JOIN_TESTS_COUNT 2
 const join_test_config_type join_test_config[JOIN_TESTS_COUNT] = { 
-  { RASQAL_JOIN_TYPE_NATURAL, 4 },
-  { RASQAL_JOIN_TYPE_LEFT, 12 },
+  { RASQAL_JOIN_TYPE_NATURAL, 2 },
+  { RASQAL_JOIN_TYPE_LEFT, 3 },
 };
 
 
@@ -712,9 +717,9 @@ main(int argc, char *argv[])
     /* vars_seq and seq are now owned by left_rs */
     vars_seq = seq = NULL;
 
-    /* 3 variables and 4 rows */
+    /* 3 variables and 2 rows */
     vars_count = 3;
-    seq = rasqal_new_row_sequence(world, vt, join_2_data_3x4_rows, vars_count,
+    seq = rasqal_new_row_sequence(world, vt, join_2_data_3x2_rows, vars_count,
                                   &vars_seq);
     if(!seq) {
       fprintf(stderr,
