@@ -71,11 +71,15 @@ static int
 rasqal_union_rowsource_init(rasqal_rowsource* rowsource, void *user_data) 
 {
   rasqal_union_rowsource_context* con;
+
   con = (rasqal_union_rowsource_context*)user_data;
   con->state = 0;
 
   con->failed = 0;
-  
+
+  rasqal_rowsource_set_requirements(con->left, RASQAL_ROWSOURCE_REQUIRE_RESET);
+  rasqal_rowsource_set_requirements(con->right, RASQAL_ROWSOURCE_REQUIRE_RESET);
+
   return 0;
 }
 
@@ -120,12 +124,12 @@ rasqal_union_rowsource_ensure_variables(rasqal_rowsource* rowsource,
     return 1;
 
   map_size = rasqal_rowsource_get_size(con->right);
-  con->right_map = (int*)RASQAL_MALLOC(int, sizeof(int) * map_size);
+  con->right_map = RASQAL_MALLOC(int*, sizeof(int) * map_size);
   if(!con->right_map)
     return 1;
 
-  con->right_tmp_values = (rasqal_literal**)RASQAL_MALLOC(ptrarray,
-                                                          sizeof(rasqal_literal*) * map_size);
+  con->right_tmp_values = RASQAL_MALLOC(rasqal_literal**,
+                                        sizeof(rasqal_literal*) * map_size);
   if(!con->right_tmp_values)
     return 1;
 
@@ -191,6 +195,15 @@ rasqal_union_rowsource_read_row(rasqal_rowsource* rowsource, void *user_data)
 
   if(con->state == 0) {
     row = rasqal_rowsource_read_row(con->left);
+#ifdef RASQAL_DEBUG
+    RASQAL_DEBUG2("rowsource %p read left row : ", rowsource);
+    if(row)
+      rasqal_row_print(row, stderr);
+    else
+      fputs("NONE", stderr);
+    fputs("\n", stderr);
+#endif
+
     if(!row)
       con->state = 1;
     else {
@@ -203,6 +216,15 @@ rasqal_union_rowsource_read_row(rasqal_rowsource* rowsource, void *user_data)
   }
   if(!row && con->state == 1) {
     row = rasqal_rowsource_read_row(con->right);
+#ifdef RASQAL_DEBUG
+    RASQAL_DEBUG2("rowsource %p read right row : ", rowsource);
+    if(row)
+      rasqal_row_print(row, stderr);
+    else
+      fputs("NONE", stderr);
+    fputs("\n", stderr);
+#endif
+
     if(!row)
       /* finished */
       con->state = 2;
@@ -313,26 +335,6 @@ rasqal_union_rowsource_reset(rasqal_rowsource* rowsource, void *user_data)
 }
 
 
-static int
-rasqal_union_rowsource_set_preserve(rasqal_rowsource* rowsource,
-                                    void *user_data, int preserve)
-{
-  rasqal_union_rowsource_context* con;
-  int rc;
-  
-  con = (rasqal_union_rowsource_context*)user_data;
-
-  con->state = 0;
-  con->failed = 0;
-
-  rc = rasqal_rowsource_set_preserve(con->left, preserve);
-  if(rc)
-    return rc;
-
-  return rasqal_rowsource_set_preserve(con->right, preserve);
-}
-
-
 static rasqal_rowsource*
 rasqal_union_rowsource_get_inner_rowsource(rasqal_rowsource* rowsource,
                                            void *user_data, int offset)
@@ -358,7 +360,7 @@ static const rasqal_rowsource_handler rasqal_union_rowsource_handler = {
   /* .read_row = */ rasqal_union_rowsource_read_row,
   /* .read_all_rows = */ rasqal_union_rowsource_read_all_rows,
   /* .reset = */ rasqal_union_rowsource_reset,
-  /* .set_preserve = */ rasqal_union_rowsource_set_preserve,
+  /* .set_requirements = */ NULL,
   /* .get_inner_rowsource = */ rasqal_union_rowsource_get_inner_rowsource,
   /* .set_origin = */ NULL,
 };
@@ -394,7 +396,7 @@ rasqal_new_union_rowsource(rasqal_world *world,
   if(!world || !query || !left || !right)
     goto fail;
   
-  con = (rasqal_union_rowsource_context*)RASQAL_CALLOC(rasqal_union_rowsource_context, 1, sizeof(rasqal_union_rowsource_context));
+  con = RASQAL_CALLOC(rasqal_union_rowsource_context*, 1, sizeof(*con));
   if(!con)
     goto fail;
 
