@@ -36,7 +36,12 @@
 #include <stdlib.h>
 #endif
 #include <stdarg.h>
+#ifdef HAVE_LIMITS_H
 #include <limits.h>
+#endif
+#ifdef HAVE_ERRNO_H
+#include <errno.h>
+#endif
 
 #include "rasqal.h"
 #include "rasqal_internal.h"
@@ -183,10 +188,9 @@ rasqal_xsd_check_double_format(const unsigned char* string, int flags)
   /* FIXME validate using
    * http://www.w3.org/TR/xmlschema-2/#double
    */
-  double d = 0.0;
   char* eptr = NULL;
 
-  d = strtod((const char*)string, &eptr);
+  (void)strtod((const char*)string, &eptr);
   if((unsigned char*)eptr != string && *eptr == '\0')
     return 1;
 
@@ -209,10 +213,9 @@ rasqal_xsd_check_float_format(const unsigned char* string, int flags)
   /* FIXME validate using
    * http://www.w3.org/TR/xmlschema-2/#float
    */
-  double d = 0.0;
   char* eptr = NULL;
 
-  d = strtod((const char*)string, &eptr);
+  (void)strtod((const char*)string, &eptr);
   if((unsigned char*)eptr != string && *eptr == '\0')
     return 1;
 
@@ -232,16 +235,17 @@ rasqal_xsd_check_float_format(const unsigned char* string, int flags)
 static int
 rasqal_xsd_check_integer_format(const unsigned char* string, int flags)
 {
-  long int v;
   char* eptr = NULL;
 
   /* This should be correct according to 
    * http://www.w3.org/TR/xmlschema-2/#integer
    */
 
-  v = (int)strtol((const char*)string, &eptr, 10);
+  errno = 0;
+  (void)strtol((const char*)string, &eptr, 10);
 
-  if((unsigned char*)eptr != string && *eptr == '\0')
+  if((unsigned char*)eptr != string && *eptr == '\0' &&
+     errno != ERANGE)
     return 1;
 
   return 0;
@@ -277,7 +281,7 @@ rasqal_xsd_format_integer(int i, size_t *len_p)
    * (the lexical form may have leading 0s in non-canonical representations)
    */
 #define INTEGER_BUFFER_SIZE 20
-  string = (unsigned char*)RASQAL_MALLOC(cstring, INTEGER_BUFFER_SIZE + 1);
+  string = RASQAL_MALLOC(unsigned char*, INTEGER_BUFFER_SIZE + 1);
   if(!string)
     return NULL;
   /* snprintf() takes as length the buffer size including NUL */
@@ -305,7 +309,7 @@ rasqal_xsd_format_float(float f, size_t *len_p)
   
   /* FIXME: This is big enough for C float formatted in decimal as %1g */
 #define FLOAT_BUFFER_SIZE 30
-  string = (unsigned char*)RASQAL_MALLOC(cstring, FLOAT_BUFFER_SIZE + 1);
+  string = RASQAL_MALLOC(unsigned char*, FLOAT_BUFFER_SIZE + 1);
   if(!string)
     return NULL;
   /* snprintf() takes as length the buffer size including NUL */
@@ -330,15 +334,15 @@ rasqal_xsd_format_float(float f, size_t *len_p)
 unsigned char*
 rasqal_xsd_format_double(double d, size_t *len_p)
 {
-  size_t e_index = 0;
+  unsigned int e_index = 0;
   int trailing_zero_start = -1;
-  size_t exponent_start;
+  unsigned int exponent_start;
   size_t len = 0;
   unsigned char* buf = NULL;
   
   if(d == 0.0f) {
     len = 5;
-    buf = (unsigned char*)RASQAL_MALLOC(cstring, len + 1);
+    buf = RASQAL_MALLOC(unsigned char*, len + 1);
     if(!buf)
       return NULL;
 
@@ -349,7 +353,7 @@ rasqal_xsd_format_double(double d, size_t *len_p)
   }
 
   len = 20;
-  buf = (unsigned char*)RASQAL_MALLOC(cstring, len + 1);
+  buf = RASQAL_MALLOC(unsigned char*, len + 1);
   if(!buf)
     return NULL;
   
@@ -360,7 +364,7 @@ rasqal_xsd_format_double(double d, size_t *len_p)
 
   for( ; buf[e_index]; ++e_index) {
     if(e_index > 0 && buf[e_index] == '0' && buf[e_index-1] != '0')
-      trailing_zero_start = e_index;
+      trailing_zero_start = (int)e_index;
     else if(buf[e_index] == 'e')
       break;
   }
@@ -377,7 +381,7 @@ rasqal_xsd_format_double(double d, size_t *len_p)
     }
   } else {
     buf[e_index] = 'E';
-    trailing_zero_start = e_index + 1;
+    trailing_zero_start = (int)e_index + 1;
   }
   
   exponent_start = e_index+2;
@@ -393,7 +397,7 @@ rasqal_xsd_format_double(double d, size_t *len_p)
     } else {
       /* copy the exponent (minus leading zeros) after the new E */
       memmove(buf + trailing_zero_start + 1, buf + exponent_start,
-              len - trailing_zero_start);
+              len - exponent_start);
       len = strlen((const char*)buf);
     }
   }
@@ -469,10 +473,7 @@ rasqal_xsd_init(rasqal_world* world)
   if(!world->xsd_namespace_uri)
     return 1;
 
-  world->xsd_datatype_uris =
-    (raptor_uri**)RASQAL_CALLOC(ptrarray,
-                                SPARQL_XSD_NAMES_COUNT + 1,
-                                sizeof(raptor_uri*));
+  world->xsd_datatype_uris = RASQAL_CALLOC(raptor_uri**, SPARQL_XSD_NAMES_COUNT + 1, sizeof(raptor_uri*));
   if(!world->xsd_datatype_uris)
     return 1;
 
