@@ -580,14 +580,14 @@ static const yytype_int8 yyrhs[] =
 /* YYRLINE[YYN] -- source line where rule number YYN was defined.  */
 static const yytype_uint16 yyrline[] =
 {
-       0,   198,   198,   202,   237,   242,   247,   256,   260,   267,
-     271,   276,   285,   290,   295,   308,   312,   316,   320,   331,
-     336,   341,   345,   349,   357,   362,   367,   372,   377,   385,
-     389,   395,   400,   406,   410,   414,   418,   422,   428,   432,
-     436,   442,   446,   450,   454,   458,   464,   471,   475,   479,
-     485,   489,   493,   497,   503,   507,   511,   517,   521,   525,
-     530,   534,   540,   544,   548,   554,   558,   564,   571,   577,
-     581,   585,   589,   593,   597,   600,   607,   612
+       0,   198,   198,   202,   246,   251,   256,   265,   269,   276,
+     280,   285,   294,   299,   304,   317,   321,   325,   329,   340,
+     345,   350,   354,   358,   366,   371,   376,   381,   386,   394,
+     398,   404,   409,   415,   419,   423,   427,   431,   437,   441,
+     445,   451,   455,   459,   463,   467,   473,   480,   484,   488,
+     494,   498,   502,   506,   512,   516,   520,   526,   530,   534,
+     539,   543,   549,   553,   557,   563,   567,   573,   580,   586,
+     590,   594,   598,   602,   606,   609,   616,   621
 };
 #endif
 
@@ -1372,7 +1372,7 @@ yydestruct (yymsg, yytype, yyvaluep)
 #line 190 "./rdql_parser.y"
 	{
   if((yyvaluep->name))
-    RASQAL_FREE(cstring, (yyvaluep->name));
+    RASQAL_FREE(char*, (yyvaluep->name));
 };
 
 /* Line 1000 of yacc.c  */
@@ -1384,7 +1384,7 @@ yydestruct (yymsg, yytype, yyvaluep)
 #line 190 "./rdql_parser.y"
 	{
   if((yyvaluep->name))
-    RASQAL_FREE(cstring, (yyvaluep->name));
+    RASQAL_FREE(char*, (yyvaluep->name));
 };
 
 /* Line 1000 of yacc.c  */
@@ -1694,13 +1694,22 @@ yyreduce:
 /* Line 1455 of yacc.c  */
 #line 203 "./rdql_parser.y"
     {
-  ((rasqal_query*)rq)->selects=(yyvsp[(2) - (7)].seq);
-  ((rasqal_query*)rq)->verb=RASQAL_QUERY_VERB_SELECT;
+  ((rasqal_query*)rq)->verb = RASQAL_QUERY_VERB_SELECT;
+
+  if((yyvsp[(2) - (7)].seq)) {
+    int i;
+
+    for(i = 0; i < raptor_sequence_size((yyvsp[(2) - (7)].seq)); i++) {
+      rasqal_variable* var = (rasqal_variable*)raptor_sequence_get_at((yyvsp[(2) - (7)].seq), i);
+      rasqal_query_add_variable((rasqal_query*)rq, var);
+    }
+    raptor_free_sequence((yyvsp[(2) - (7)].seq));
+  }
 
   if((yyvsp[(3) - (7)].seq)) {
     int i;
     
-    for(i=0; i < raptor_sequence_size((yyvsp[(3) - (7)].seq)); i++) {
+    for(i = 0; i < raptor_sequence_size((yyvsp[(3) - (7)].seq)); i++) {
       raptor_uri* uri = (raptor_uri*)raptor_sequence_get_at((yyvsp[(3) - (7)].seq), i);
       rasqal_data_graph* dg;
       
@@ -1730,7 +1739,7 @@ yyreduce:
   case 4:
 
 /* Line 1455 of yacc.c  */
-#line 238 "./rdql_parser.y"
+#line 247 "./rdql_parser.y"
     {
   (yyval.seq)=(yyvsp[(1) - (3)].seq);
   raptor_sequence_push((yyval.seq), (yyvsp[(3) - (3)].variable));
@@ -1740,7 +1749,7 @@ yyreduce:
   case 5:
 
 /* Line 1455 of yacc.c  */
-#line 243 "./rdql_parser.y"
+#line 252 "./rdql_parser.y"
     {
   (yyval.seq)=(yyvsp[(1) - (2)].seq);
   raptor_sequence_push((yyval.seq), (yyvsp[(2) - (2)].variable));
@@ -1750,7 +1759,7 @@ yyreduce:
   case 6:
 
 /* Line 1455 of yacc.c  */
-#line 248 "./rdql_parser.y"
+#line 257 "./rdql_parser.y"
     {
   (yyval.seq) = raptor_new_sequence((raptor_data_free_handler)rasqal_free_variable,
                            (raptor_data_print_handler)rasqal_variable_print);
@@ -1761,7 +1770,7 @@ yyreduce:
   case 7:
 
 /* Line 1455 of yacc.c  */
-#line 257 "./rdql_parser.y"
+#line 266 "./rdql_parser.y"
     {
   (yyval.seq)=(yyvsp[(1) - (1)].seq);
 }
@@ -1770,17 +1779,17 @@ yyreduce:
   case 8:
 
 /* Line 1455 of yacc.c  */
-#line 261 "./rdql_parser.y"
+#line 270 "./rdql_parser.y"
     {
   (yyval.seq)=NULL;
-  ((rasqal_query*)rq)->wildcard=1;
+  rasqal_query_set_wildcard((rasqal_query*)rq, 1);
 }
     break;
 
   case 9:
 
 /* Line 1455 of yacc.c  */
-#line 268 "./rdql_parser.y"
+#line 277 "./rdql_parser.y"
     {
   (yyval.seq)=(yyvsp[(2) - (2)].seq);
 }
@@ -1789,7 +1798,7 @@ yyreduce:
   case 10:
 
 /* Line 1455 of yacc.c  */
-#line 272 "./rdql_parser.y"
+#line 281 "./rdql_parser.y"
     {
   (yyval.seq)=(yyvsp[(2) - (2)].seq);
 }
@@ -1798,7 +1807,7 @@ yyreduce:
   case 11:
 
 /* Line 1455 of yacc.c  */
-#line 276 "./rdql_parser.y"
+#line 285 "./rdql_parser.y"
     {
   (yyval.seq)=NULL;
 }
@@ -1807,7 +1816,7 @@ yyreduce:
   case 12:
 
 /* Line 1455 of yacc.c  */
-#line 286 "./rdql_parser.y"
+#line 295 "./rdql_parser.y"
     {
   (yyval.seq)=(yyvsp[(1) - (3)].seq);
   raptor_sequence_push((yyval.seq), (yyvsp[(3) - (3)].triple));
@@ -1817,7 +1826,7 @@ yyreduce:
   case 13:
 
 /* Line 1455 of yacc.c  */
-#line 291 "./rdql_parser.y"
+#line 300 "./rdql_parser.y"
     {
   (yyval.seq)=(yyvsp[(1) - (2)].seq);
   raptor_sequence_push((yyval.seq), (yyvsp[(2) - (2)].triple));
@@ -1827,7 +1836,7 @@ yyreduce:
   case 14:
 
 /* Line 1455 of yacc.c  */
-#line 296 "./rdql_parser.y"
+#line 305 "./rdql_parser.y"
     {
   (yyval.seq)=((rasqal_query*)rq)->triples;
   raptor_sequence_push((yyval.seq), (yyvsp[(1) - (1)].triple));
@@ -1837,7 +1846,7 @@ yyreduce:
   case 15:
 
 /* Line 1455 of yacc.c  */
-#line 309 "./rdql_parser.y"
+#line 318 "./rdql_parser.y"
     {
   (yyval.triple)=rasqal_new_triple((yyvsp[(2) - (7)].literal), (yyvsp[(4) - (7)].literal), (yyvsp[(6) - (7)].literal));
 }
@@ -1846,7 +1855,7 @@ yyreduce:
   case 16:
 
 /* Line 1455 of yacc.c  */
-#line 313 "./rdql_parser.y"
+#line 322 "./rdql_parser.y"
     {
   (yyval.triple)=rasqal_new_triple((yyvsp[(2) - (6)].literal), (yyvsp[(3) - (6)].literal), (yyvsp[(5) - (6)].literal));
 }
@@ -1855,7 +1864,7 @@ yyreduce:
   case 17:
 
 /* Line 1455 of yacc.c  */
-#line 317 "./rdql_parser.y"
+#line 326 "./rdql_parser.y"
     {
   (yyval.triple)=rasqal_new_triple((yyvsp[(2) - (6)].literal), (yyvsp[(4) - (6)].literal), (yyvsp[(5) - (6)].literal));
 }
@@ -1864,7 +1873,7 @@ yyreduce:
   case 18:
 
 /* Line 1455 of yacc.c  */
-#line 321 "./rdql_parser.y"
+#line 330 "./rdql_parser.y"
     {
   (yyval.triple)=rasqal_new_triple((yyvsp[(2) - (5)].literal), (yyvsp[(3) - (5)].literal), (yyvsp[(4) - (5)].literal));
 }
@@ -1873,7 +1882,7 @@ yyreduce:
   case 19:
 
 /* Line 1455 of yacc.c  */
-#line 332 "./rdql_parser.y"
+#line 341 "./rdql_parser.y"
     {
   (yyval.expr)=(yyvsp[(2) - (2)].expr);
 }
@@ -1882,7 +1891,7 @@ yyreduce:
   case 20:
 
 /* Line 1455 of yacc.c  */
-#line 336 "./rdql_parser.y"
+#line 345 "./rdql_parser.y"
     {
   (yyval.expr)=NULL;
 }
@@ -1891,7 +1900,7 @@ yyreduce:
   case 21:
 
 /* Line 1455 of yacc.c  */
-#line 342 "./rdql_parser.y"
+#line 351 "./rdql_parser.y"
     {
   (yyval.expr)=rasqal_new_2op_expression(((rasqal_query*)rq)->world, RASQAL_EXPR_AND, (yyvsp[(1) - (3)].expr), (yyvsp[(3) - (3)].expr));
 }
@@ -1900,7 +1909,7 @@ yyreduce:
   case 22:
 
 /* Line 1455 of yacc.c  */
-#line 346 "./rdql_parser.y"
+#line 355 "./rdql_parser.y"
     {
   (yyval.expr)=rasqal_new_2op_expression(((rasqal_query*)rq)->world, RASQAL_EXPR_AND, (yyvsp[(1) - (3)].expr), (yyvsp[(3) - (3)].expr));
 }
@@ -1909,7 +1918,7 @@ yyreduce:
   case 23:
 
 /* Line 1455 of yacc.c  */
-#line 350 "./rdql_parser.y"
+#line 359 "./rdql_parser.y"
     {
   (yyval.expr)=(yyvsp[(1) - (1)].expr);
 }
@@ -1918,7 +1927,7 @@ yyreduce:
   case 24:
 
 /* Line 1455 of yacc.c  */
-#line 358 "./rdql_parser.y"
+#line 367 "./rdql_parser.y"
     {
   (yyval.seq)=(yyvsp[(2) - (2)].seq);
 }
@@ -1927,7 +1936,7 @@ yyreduce:
   case 25:
 
 /* Line 1455 of yacc.c  */
-#line 362 "./rdql_parser.y"
+#line 371 "./rdql_parser.y"
     {
   (yyval.seq)=NULL;
 }
@@ -1936,7 +1945,7 @@ yyreduce:
   case 26:
 
 /* Line 1455 of yacc.c  */
-#line 368 "./rdql_parser.y"
+#line 377 "./rdql_parser.y"
     {
   (yyval.seq)=((rasqal_query*)rq)->prefixes;
   raptor_sequence_shift((yyval.seq), rasqal_new_prefix(((rasqal_query*)rq)->world, (yyvsp[(1) - (5)].name), (yyvsp[(3) - (5)].uri)));
@@ -1946,7 +1955,7 @@ yyreduce:
   case 27:
 
 /* Line 1455 of yacc.c  */
-#line 373 "./rdql_parser.y"
+#line 382 "./rdql_parser.y"
     {
   (yyval.seq)=((rasqal_query*)rq)->prefixes;
   raptor_sequence_shift((yyval.seq), rasqal_new_prefix(((rasqal_query*)rq)->world, (yyvsp[(1) - (4)].name), (yyvsp[(3) - (4)].uri)));
@@ -1956,7 +1965,7 @@ yyreduce:
   case 28:
 
 /* Line 1455 of yacc.c  */
-#line 378 "./rdql_parser.y"
+#line 387 "./rdql_parser.y"
     {
   (yyval.seq)=((rasqal_query*)rq)->prefixes;
   raptor_sequence_push((yyval.seq), rasqal_new_prefix(((rasqal_query*)rq)->world, (yyvsp[(1) - (3)].name), (yyvsp[(3) - (3)].uri)));
@@ -1966,7 +1975,7 @@ yyreduce:
   case 29:
 
 /* Line 1455 of yacc.c  */
-#line 386 "./rdql_parser.y"
+#line 395 "./rdql_parser.y"
     {
   (yyval.expr)=rasqal_new_2op_expression(((rasqal_query*)rq)->world, RASQAL_EXPR_OR, (yyvsp[(1) - (3)].expr), (yyvsp[(3) - (3)].expr));
 }
@@ -1975,7 +1984,7 @@ yyreduce:
   case 30:
 
 /* Line 1455 of yacc.c  */
-#line 390 "./rdql_parser.y"
+#line 399 "./rdql_parser.y"
     {
   (yyval.expr)=(yyvsp[(1) - (1)].expr);
 }
@@ -1984,7 +1993,7 @@ yyreduce:
   case 31:
 
 /* Line 1455 of yacc.c  */
-#line 396 "./rdql_parser.y"
+#line 405 "./rdql_parser.y"
     {
   (yyval.expr)=rasqal_new_2op_expression(((rasqal_query*)rq)->world, RASQAL_EXPR_AND, (yyvsp[(1) - (3)].expr), (yyvsp[(3) - (3)].expr));
 ;
@@ -1994,7 +2003,7 @@ yyreduce:
   case 32:
 
 /* Line 1455 of yacc.c  */
-#line 401 "./rdql_parser.y"
+#line 410 "./rdql_parser.y"
     {
   (yyval.expr)=(yyvsp[(1) - (1)].expr);
 }
@@ -2003,7 +2012,7 @@ yyreduce:
   case 33:
 
 /* Line 1455 of yacc.c  */
-#line 407 "./rdql_parser.y"
+#line 416 "./rdql_parser.y"
     {
   (yyval.expr)=rasqal_new_2op_expression(((rasqal_query*)rq)->world, RASQAL_EXPR_STR_EQ, (yyvsp[(1) - (3)].expr), (yyvsp[(3) - (3)].expr));
 }
@@ -2012,7 +2021,7 @@ yyreduce:
   case 34:
 
 /* Line 1455 of yacc.c  */
-#line 411 "./rdql_parser.y"
+#line 420 "./rdql_parser.y"
     {
   (yyval.expr)=rasqal_new_2op_expression(((rasqal_query*)rq)->world, RASQAL_EXPR_STR_NEQ, (yyvsp[(1) - (3)].expr), (yyvsp[(3) - (3)].expr));
 }
@@ -2021,7 +2030,7 @@ yyreduce:
   case 35:
 
 /* Line 1455 of yacc.c  */
-#line 415 "./rdql_parser.y"
+#line 424 "./rdql_parser.y"
     {
   (yyval.expr)=rasqal_new_string_op_expression(((rasqal_query*)rq)->world, RASQAL_EXPR_STR_MATCH, (yyvsp[(1) - (3)].expr), (yyvsp[(3) - (3)].literal));
 }
@@ -2030,7 +2039,7 @@ yyreduce:
   case 36:
 
 /* Line 1455 of yacc.c  */
-#line 419 "./rdql_parser.y"
+#line 428 "./rdql_parser.y"
     {
   (yyval.expr)=rasqal_new_string_op_expression(((rasqal_query*)rq)->world, RASQAL_EXPR_STR_NMATCH, (yyvsp[(1) - (3)].expr), (yyvsp[(3) - (3)].literal));
 }
@@ -2039,7 +2048,7 @@ yyreduce:
   case 37:
 
 /* Line 1455 of yacc.c  */
-#line 423 "./rdql_parser.y"
+#line 432 "./rdql_parser.y"
     {
   (yyval.expr)=(yyvsp[(1) - (1)].expr);
 }
@@ -2048,7 +2057,7 @@ yyreduce:
   case 38:
 
 /* Line 1455 of yacc.c  */
-#line 429 "./rdql_parser.y"
+#line 438 "./rdql_parser.y"
     {
   (yyval.expr)=rasqal_new_2op_expression(((rasqal_query*)rq)->world, RASQAL_EXPR_EQ, (yyvsp[(1) - (3)].expr), (yyvsp[(3) - (3)].expr));
 }
@@ -2057,7 +2066,7 @@ yyreduce:
   case 39:
 
 /* Line 1455 of yacc.c  */
-#line 433 "./rdql_parser.y"
+#line 442 "./rdql_parser.y"
     {
   (yyval.expr)=rasqal_new_2op_expression(((rasqal_query*)rq)->world, RASQAL_EXPR_NEQ, (yyvsp[(1) - (3)].expr), (yyvsp[(3) - (3)].expr));
 }
@@ -2066,7 +2075,7 @@ yyreduce:
   case 40:
 
 /* Line 1455 of yacc.c  */
-#line 437 "./rdql_parser.y"
+#line 446 "./rdql_parser.y"
     {
   (yyval.expr)=(yyvsp[(1) - (1)].expr);
 }
@@ -2075,7 +2084,7 @@ yyreduce:
   case 41:
 
 /* Line 1455 of yacc.c  */
-#line 443 "./rdql_parser.y"
+#line 452 "./rdql_parser.y"
     {
   (yyval.expr)=rasqal_new_2op_expression(((rasqal_query*)rq)->world, RASQAL_EXPR_LT, (yyvsp[(1) - (3)].expr), (yyvsp[(3) - (3)].expr));
 }
@@ -2084,7 +2093,7 @@ yyreduce:
   case 42:
 
 /* Line 1455 of yacc.c  */
-#line 447 "./rdql_parser.y"
+#line 456 "./rdql_parser.y"
     {
   (yyval.expr)=rasqal_new_2op_expression(((rasqal_query*)rq)->world, RASQAL_EXPR_GT, (yyvsp[(1) - (3)].expr), (yyvsp[(3) - (3)].expr));
 }
@@ -2093,7 +2102,7 @@ yyreduce:
   case 43:
 
 /* Line 1455 of yacc.c  */
-#line 451 "./rdql_parser.y"
+#line 460 "./rdql_parser.y"
     {
   (yyval.expr)=rasqal_new_2op_expression(((rasqal_query*)rq)->world, RASQAL_EXPR_LE, (yyvsp[(1) - (3)].expr), (yyvsp[(3) - (3)].expr));
 }
@@ -2102,7 +2111,7 @@ yyreduce:
   case 44:
 
 /* Line 1455 of yacc.c  */
-#line 455 "./rdql_parser.y"
+#line 464 "./rdql_parser.y"
     {
   (yyval.expr)=rasqal_new_2op_expression(((rasqal_query*)rq)->world, RASQAL_EXPR_GE, (yyvsp[(1) - (3)].expr), (yyvsp[(3) - (3)].expr));
 }
@@ -2111,7 +2120,7 @@ yyreduce:
   case 45:
 
 /* Line 1455 of yacc.c  */
-#line 459 "./rdql_parser.y"
+#line 468 "./rdql_parser.y"
     {
   (yyval.expr)=(yyvsp[(1) - (1)].expr);
 }
@@ -2120,7 +2129,7 @@ yyreduce:
   case 46:
 
 /* Line 1455 of yacc.c  */
-#line 465 "./rdql_parser.y"
+#line 474 "./rdql_parser.y"
     {
   (yyval.expr)=(yyvsp[(1) - (1)].expr);
 }
@@ -2129,7 +2138,7 @@ yyreduce:
   case 47:
 
 /* Line 1455 of yacc.c  */
-#line 472 "./rdql_parser.y"
+#line 481 "./rdql_parser.y"
     {
   (yyval.expr)=rasqal_new_2op_expression(((rasqal_query*)rq)->world, RASQAL_EXPR_PLUS, (yyvsp[(1) - (3)].expr), (yyvsp[(3) - (3)].expr));
 }
@@ -2138,7 +2147,7 @@ yyreduce:
   case 48:
 
 /* Line 1455 of yacc.c  */
-#line 476 "./rdql_parser.y"
+#line 485 "./rdql_parser.y"
     {
   (yyval.expr)=rasqal_new_2op_expression(((rasqal_query*)rq)->world, RASQAL_EXPR_MINUS, (yyvsp[(1) - (3)].expr), (yyvsp[(3) - (3)].expr));
 }
@@ -2147,7 +2156,7 @@ yyreduce:
   case 49:
 
 /* Line 1455 of yacc.c  */
-#line 480 "./rdql_parser.y"
+#line 489 "./rdql_parser.y"
     {
   (yyval.expr)=(yyvsp[(1) - (1)].expr);
 }
@@ -2156,7 +2165,7 @@ yyreduce:
   case 50:
 
 /* Line 1455 of yacc.c  */
-#line 486 "./rdql_parser.y"
+#line 495 "./rdql_parser.y"
     {
   (yyval.expr)=rasqal_new_2op_expression(((rasqal_query*)rq)->world, RASQAL_EXPR_STAR, (yyvsp[(1) - (3)].expr), (yyvsp[(3) - (3)].expr));
 }
@@ -2165,7 +2174,7 @@ yyreduce:
   case 51:
 
 /* Line 1455 of yacc.c  */
-#line 490 "./rdql_parser.y"
+#line 499 "./rdql_parser.y"
     {
   (yyval.expr)=rasqal_new_2op_expression(((rasqal_query*)rq)->world, RASQAL_EXPR_SLASH, (yyvsp[(1) - (3)].expr), (yyvsp[(3) - (3)].expr));
 }
@@ -2174,7 +2183,7 @@ yyreduce:
   case 52:
 
 /* Line 1455 of yacc.c  */
-#line 494 "./rdql_parser.y"
+#line 503 "./rdql_parser.y"
     {
   (yyval.expr)=rasqal_new_2op_expression(((rasqal_query*)rq)->world, RASQAL_EXPR_REM, (yyvsp[(1) - (3)].expr), (yyvsp[(3) - (3)].expr));
 }
@@ -2183,7 +2192,7 @@ yyreduce:
   case 53:
 
 /* Line 1455 of yacc.c  */
-#line 498 "./rdql_parser.y"
+#line 507 "./rdql_parser.y"
     {
   (yyval.expr)=(yyvsp[(1) - (1)].expr);
 }
@@ -2192,7 +2201,7 @@ yyreduce:
   case 54:
 
 /* Line 1455 of yacc.c  */
-#line 504 "./rdql_parser.y"
+#line 513 "./rdql_parser.y"
     {
   (yyval.expr)=(yyvsp[(2) - (2)].expr);
 }
@@ -2201,7 +2210,7 @@ yyreduce:
   case 55:
 
 /* Line 1455 of yacc.c  */
-#line 508 "./rdql_parser.y"
+#line 517 "./rdql_parser.y"
     {
   (yyval.expr)=rasqal_new_1op_expression(((rasqal_query*)rq)->world, RASQAL_EXPR_UMINUS, (yyvsp[(2) - (2)].expr));
 }
@@ -2210,7 +2219,7 @@ yyreduce:
   case 56:
 
 /* Line 1455 of yacc.c  */
-#line 512 "./rdql_parser.y"
+#line 521 "./rdql_parser.y"
     {
   (yyval.expr)=(yyvsp[(1) - (1)].expr);
 }
@@ -2219,7 +2228,7 @@ yyreduce:
   case 57:
 
 /* Line 1455 of yacc.c  */
-#line 518 "./rdql_parser.y"
+#line 527 "./rdql_parser.y"
     {
   (yyval.expr)=rasqal_new_1op_expression(((rasqal_query*)rq)->world, RASQAL_EXPR_TILDE, (yyvsp[(2) - (2)].expr));
 }
@@ -2228,7 +2237,7 @@ yyreduce:
   case 58:
 
 /* Line 1455 of yacc.c  */
-#line 522 "./rdql_parser.y"
+#line 531 "./rdql_parser.y"
     {
   (yyval.expr)=rasqal_new_1op_expression(((rasqal_query*)rq)->world, RASQAL_EXPR_BANG, (yyvsp[(2) - (2)].expr));
 }
@@ -2237,7 +2246,7 @@ yyreduce:
   case 59:
 
 /* Line 1455 of yacc.c  */
-#line 526 "./rdql_parser.y"
+#line 535 "./rdql_parser.y"
     {
   rasqal_literal *l=rasqal_new_variable_literal(((rasqal_query*)rq)->world, (yyvsp[(1) - (1)].variable));
   (yyval.expr)=rasqal_new_literal_expression(((rasqal_query*)rq)->world, l);
@@ -2247,7 +2256,7 @@ yyreduce:
   case 60:
 
 /* Line 1455 of yacc.c  */
-#line 531 "./rdql_parser.y"
+#line 540 "./rdql_parser.y"
     {
   (yyval.expr)=rasqal_new_literal_expression(((rasqal_query*)rq)->world, (yyvsp[(1) - (1)].literal));
 }
@@ -2256,7 +2265,7 @@ yyreduce:
   case 61:
 
 /* Line 1455 of yacc.c  */
-#line 535 "./rdql_parser.y"
+#line 544 "./rdql_parser.y"
     {
   (yyval.expr)=(yyvsp[(2) - (3)].expr);
 }
@@ -2265,7 +2274,7 @@ yyreduce:
   case 62:
 
 /* Line 1455 of yacc.c  */
-#line 541 "./rdql_parser.y"
+#line 550 "./rdql_parser.y"
     {
   (yyval.literal)=rasqal_new_variable_literal(((rasqal_query*)rq)->world, (yyvsp[(1) - (1)].variable));
 }
@@ -2274,7 +2283,7 @@ yyreduce:
   case 63:
 
 /* Line 1455 of yacc.c  */
-#line 545 "./rdql_parser.y"
+#line 554 "./rdql_parser.y"
     {
   (yyval.literal)=rasqal_new_uri_literal(((rasqal_query*)rq)->world, (yyvsp[(1) - (1)].uri));
 }
@@ -2283,7 +2292,7 @@ yyreduce:
   case 64:
 
 /* Line 1455 of yacc.c  */
-#line 549 "./rdql_parser.y"
+#line 558 "./rdql_parser.y"
     {
   (yyval.literal)=rasqal_new_simple_literal(((rasqal_query*)rq)->world, RASQAL_LITERAL_QNAME, (yyvsp[(1) - (1)].name));
 }
@@ -2292,7 +2301,7 @@ yyreduce:
   case 65:
 
 /* Line 1455 of yacc.c  */
-#line 555 "./rdql_parser.y"
+#line 564 "./rdql_parser.y"
     {
   (yyval.literal)=rasqal_new_variable_literal(((rasqal_query*)rq)->world, (yyvsp[(1) - (1)].variable));
 }
@@ -2301,7 +2310,7 @@ yyreduce:
   case 66:
 
 /* Line 1455 of yacc.c  */
-#line 559 "./rdql_parser.y"
+#line 568 "./rdql_parser.y"
     {
   (yyval.literal)=(yyvsp[(1) - (1)].literal);
 }
@@ -2310,7 +2319,7 @@ yyreduce:
   case 67:
 
 /* Line 1455 of yacc.c  */
-#line 565 "./rdql_parser.y"
+#line 574 "./rdql_parser.y"
     {
   (yyval.variable) = rasqal_variables_table_add(((rasqal_query*)rq)->vars_table,
                                   RASQAL_VARIABLE_TYPE_NORMAL, (yyvsp[(2) - (2)].name), NULL);
@@ -2320,7 +2329,7 @@ yyreduce:
   case 68:
 
 /* Line 1455 of yacc.c  */
-#line 572 "./rdql_parser.y"
+#line 581 "./rdql_parser.y"
     {
   (yyval.literal)=(yyvsp[(1) - (1)].literal);
 }
@@ -2329,7 +2338,7 @@ yyreduce:
   case 69:
 
 /* Line 1455 of yacc.c  */
-#line 578 "./rdql_parser.y"
+#line 587 "./rdql_parser.y"
     {
   (yyval.literal)=rasqal_new_uri_literal(((rasqal_query*)rq)->world, (yyvsp[(1) - (1)].uri));
 }
@@ -2338,7 +2347,7 @@ yyreduce:
   case 70:
 
 /* Line 1455 of yacc.c  */
-#line 582 "./rdql_parser.y"
+#line 591 "./rdql_parser.y"
     {
   (yyval.literal)=(yyvsp[(1) - (1)].literal);
 }
@@ -2347,7 +2356,7 @@ yyreduce:
   case 71:
 
 /* Line 1455 of yacc.c  */
-#line 586 "./rdql_parser.y"
+#line 595 "./rdql_parser.y"
     {
   (yyval.literal)=(yyvsp[(1) - (1)].literal);
 }
@@ -2356,7 +2365,7 @@ yyreduce:
   case 72:
 
 /* Line 1455 of yacc.c  */
-#line 590 "./rdql_parser.y"
+#line 599 "./rdql_parser.y"
     {
   (yyval.literal)=(yyvsp[(1) - (1)].literal);
 }
@@ -2365,7 +2374,7 @@ yyreduce:
   case 73:
 
 /* Line 1455 of yacc.c  */
-#line 594 "./rdql_parser.y"
+#line 603 "./rdql_parser.y"
     {
   (yyval.literal)=(yyvsp[(1) - (1)].literal);
 }
@@ -2374,7 +2383,7 @@ yyreduce:
   case 74:
 
 /* Line 1455 of yacc.c  */
-#line 598 "./rdql_parser.y"
+#line 607 "./rdql_parser.y"
     {
   (yyval.literal)=(yyvsp[(1) - (1)].literal);
 }
@@ -2383,7 +2392,7 @@ yyreduce:
   case 75:
 
 /* Line 1455 of yacc.c  */
-#line 601 "./rdql_parser.y"
+#line 610 "./rdql_parser.y"
     {
   (yyval.literal)=rasqal_new_simple_literal(((rasqal_query*)rq)->world, RASQAL_LITERAL_QNAME, (yyvsp[(1) - (1)].name));
 }
@@ -2392,7 +2401,7 @@ yyreduce:
   case 76:
 
 /* Line 1455 of yacc.c  */
-#line 608 "./rdql_parser.y"
+#line 617 "./rdql_parser.y"
     {
   (yyval.seq)=(yyvsp[(3) - (3)].seq);
   raptor_sequence_shift((yyval.seq), (yyvsp[(1) - (3)].uri));
@@ -2402,7 +2411,7 @@ yyreduce:
   case 77:
 
 /* Line 1455 of yacc.c  */
-#line 613 "./rdql_parser.y"
+#line 622 "./rdql_parser.y"
     {
   (yyval.seq) = raptor_new_sequence((raptor_data_free_handler)raptor_free_uri,
                            (raptor_data_print_handler)raptor_uri_print);
@@ -2413,7 +2422,7 @@ yyreduce:
 
 
 /* Line 1455 of yacc.c  */
-#line 2417 "rdql_parser.c"
+#line 2426 "rdql_parser.c"
       default: break;
     }
   YY_SYMBOL_PRINT ("-> $$ =", yyr1[yyn], &yyval, &yyloc);
@@ -2625,7 +2634,7 @@ yyreturn:
 
 
 /* Line 1675 of yacc.c  */
-#line 620 "./rdql_parser.y"
+#line 629 "./rdql_parser.y"
 
 
 
@@ -2725,7 +2734,8 @@ rasqal_rdql_query_language_prepare(rasqal_query* rdf_query) {
     return 1;
 
   /* RDQL: Expand 'SELECT *' */
-  if(rasqal_query_expand_wildcards(rdf_query))
+  if(rasqal_query_expand_wildcards(rdf_query,
+                                   rasqal_query_get_projection(rdf_query)))
     return 1;
   
   return 0;
@@ -2736,7 +2746,6 @@ static int
 rdql_parse(rasqal_query* rq) {
   rasqal_rdql_query_language* rqe=(rasqal_rdql_query_language*)rq->context;
   raptor_locator *locator=&rq->locator;
-  void *buffer;
   
   if(!rq->query_string)
     return yy_init_globals(NULL); /* 0 but a way to use yy_init_globals */
@@ -2756,7 +2765,7 @@ rdql_parse(rasqal_query* rq) {
 
   rdql_lexer_set_extra(((rasqal_query*)rq), rqe->scanner);
 
-  buffer= rdql_lexer__scan_buffer((char*)rq->query_string, rq->query_string_length, rqe->scanner);
+  (void)rdql_lexer__scan_buffer((char*)rq->query_string, rq->query_string_length, rqe->scanner);
 
   rqe->error_count=0;
 
@@ -2820,22 +2829,25 @@ rdql_syntax_error(rasqal_query *rq, const char *message, ...)
 int
 rdql_syntax_warning(rasqal_query *rq, const char *message, ...)
 {
-  rasqal_rdql_query_language *rqe=(rasqal_rdql_query_language*)rq->context;
+  rasqal_rdql_query_language *rqe;
   va_list arguments;
 
-  rq->locator.line=rqe->lineno;
+  if(RASQAL_WARNING_LEVEL_QUERY_SYNTAX < rq->world->warning_level)
+    return 0;
+  
+  rqe = (rasqal_rdql_query_language*)rq->context;
+
+  rq->locator.line = rqe->lineno;
 #ifdef RASQAL_RDQL_USE_ERROR_COLUMNS
   /*  rq->locator.column=rdql_lexer_get_column(yyscanner);*/
 #endif
 
   va_start(arguments, message);
-  rasqal_log_error_varargs(rq->world,
-                           RAPTOR_LOG_LEVEL_WARN,
-                           &rq->locator,
-                           message, arguments);
+  rasqal_log_error_varargs(rq->world, RAPTOR_LOG_LEVEL_WARN,
+                           &rq->locator, message, arguments);
   va_end(arguments);
 
-   return (0);
+  return 0;
 }
 
 
