@@ -129,52 +129,55 @@
      CONTAINS = 345,
      ENCODE_FOR_URI = 346,
      CONCAT = 347,
-     BIND = 348,
-     ABS = 349,
-     ROUND = 350,
-     CEIL = 351,
-     FLOOR = 352,
-     RAND = 353,
-     MD5 = 354,
-     SHA1 = 355,
-     SHA224 = 356,
-     SHA256 = 357,
-     SHA384 = 358,
-     SHA512 = 359,
-     EXPLAIN = 360,
-     LET = 361,
-     CURRENT_DATETIME = 362,
-     NOW = 363,
-     FROM_UNIXTIME = 364,
-     TO_UNIXTIME = 365,
-     HATHAT = 366,
-     SC_OR = 367,
-     SC_AND = 368,
-     EQ = 369,
-     NEQ = 370,
-     LT = 371,
-     GT = 372,
-     LE = 373,
-     GE = 374,
-     ASSIGN = 375,
-     STRING = 376,
-     LANG_TAG = 377,
-     DOUBLE_LITERAL = 378,
-     DOUBLE_POSITIVE_LITERAL = 379,
-     DOUBLE_NEGATIVE_LITERAL = 380,
-     INTEGER_LITERAL = 381,
-     INTEGER_POSITIVE_LITERAL = 382,
-     INTEGER_NEGATIVE_LITERAL = 383,
-     DECIMAL_LITERAL = 384,
-     DECIMAL_POSITIVE_LITERAL = 385,
-     DECIMAL_NEGATIVE_LITERAL = 386,
-     BOOLEAN_LITERAL = 387,
-     URI_LITERAL = 388,
-     URI_LITERAL_BRACE = 389,
-     QNAME_LITERAL = 390,
-     QNAME_LITERAL_BRACE = 391,
-     BLANK_LITERAL = 392,
-     IDENTIFIER = 393
+     STRBEFORE = 348,
+     STRAFTER = 349,
+     REPLACE = 350,
+     BIND = 351,
+     ABS = 352,
+     ROUND = 353,
+     CEIL = 354,
+     FLOOR = 355,
+     RAND = 356,
+     MD5 = 357,
+     SHA1 = 358,
+     SHA224 = 359,
+     SHA256 = 360,
+     SHA384 = 361,
+     SHA512 = 362,
+     EXPLAIN = 363,
+     LET = 364,
+     CURRENT_DATETIME = 365,
+     NOW = 366,
+     FROM_UNIXTIME = 367,
+     TO_UNIXTIME = 368,
+     HATHAT = 369,
+     SC_OR = 370,
+     SC_AND = 371,
+     EQ = 372,
+     NEQ = 373,
+     LT = 374,
+     GT = 375,
+     LE = 376,
+     GE = 377,
+     ASSIGN = 378,
+     STRING = 379,
+     LANG_TAG = 380,
+     DOUBLE_LITERAL = 381,
+     DOUBLE_POSITIVE_LITERAL = 382,
+     DOUBLE_NEGATIVE_LITERAL = 383,
+     INTEGER_LITERAL = 384,
+     INTEGER_POSITIVE_LITERAL = 385,
+     INTEGER_NEGATIVE_LITERAL = 386,
+     DECIMAL_LITERAL = 387,
+     DECIMAL_POSITIVE_LITERAL = 388,
+     DECIMAL_NEGATIVE_LITERAL = 389,
+     BOOLEAN_LITERAL = 390,
+     URI_LITERAL = 391,
+     URI_LITERAL_BRACE = 392,
+     QNAME_LITERAL = 393,
+     QNAME_LITERAL_BRACE = 394,
+     BLANK_LITERAL = 395,
+     IDENTIFIER = 396
    };
 #endif
 /* Tokens.  */
@@ -268,52 +271,55 @@
 #define CONTAINS 345
 #define ENCODE_FOR_URI 346
 #define CONCAT 347
-#define BIND 348
-#define ABS 349
-#define ROUND 350
-#define CEIL 351
-#define FLOOR 352
-#define RAND 353
-#define MD5 354
-#define SHA1 355
-#define SHA224 356
-#define SHA256 357
-#define SHA384 358
-#define SHA512 359
-#define EXPLAIN 360
-#define LET 361
-#define CURRENT_DATETIME 362
-#define NOW 363
-#define FROM_UNIXTIME 364
-#define TO_UNIXTIME 365
-#define HATHAT 366
-#define SC_OR 367
-#define SC_AND 368
-#define EQ 369
-#define NEQ 370
-#define LT 371
-#define GT 372
-#define LE 373
-#define GE 374
-#define ASSIGN 375
-#define STRING 376
-#define LANG_TAG 377
-#define DOUBLE_LITERAL 378
-#define DOUBLE_POSITIVE_LITERAL 379
-#define DOUBLE_NEGATIVE_LITERAL 380
-#define INTEGER_LITERAL 381
-#define INTEGER_POSITIVE_LITERAL 382
-#define INTEGER_NEGATIVE_LITERAL 383
-#define DECIMAL_LITERAL 384
-#define DECIMAL_POSITIVE_LITERAL 385
-#define DECIMAL_NEGATIVE_LITERAL 386
-#define BOOLEAN_LITERAL 387
-#define URI_LITERAL 388
-#define URI_LITERAL_BRACE 389
-#define QNAME_LITERAL 390
-#define QNAME_LITERAL_BRACE 391
-#define BLANK_LITERAL 392
-#define IDENTIFIER 393
+#define STRBEFORE 348
+#define STRAFTER 349
+#define REPLACE 350
+#define BIND 351
+#define ABS 352
+#define ROUND 353
+#define CEIL 354
+#define FLOOR 355
+#define RAND 356
+#define MD5 357
+#define SHA1 358
+#define SHA224 359
+#define SHA256 360
+#define SHA384 361
+#define SHA512 362
+#define EXPLAIN 363
+#define LET 364
+#define CURRENT_DATETIME 365
+#define NOW 366
+#define FROM_UNIXTIME 367
+#define TO_UNIXTIME 368
+#define HATHAT 369
+#define SC_OR 370
+#define SC_AND 371
+#define EQ 372
+#define NEQ 373
+#define LT 374
+#define GT 375
+#define LE 376
+#define GE 377
+#define ASSIGN 378
+#define STRING 379
+#define LANG_TAG 380
+#define DOUBLE_LITERAL 381
+#define DOUBLE_POSITIVE_LITERAL 382
+#define DOUBLE_NEGATIVE_LITERAL 383
+#define INTEGER_LITERAL 384
+#define INTEGER_POSITIVE_LITERAL 385
+#define INTEGER_NEGATIVE_LITERAL 386
+#define DECIMAL_LITERAL 387
+#define DECIMAL_POSITIVE_LITERAL 388
+#define DECIMAL_NEGATIVE_LITERAL 389
+#define BOOLEAN_LITERAL 390
+#define URI_LITERAL 391
+#define URI_LITERAL_BRACE 392
+#define QNAME_LITERAL 393
+#define QNAME_LITERAL_BRACE 394
+#define BLANK_LITERAL 395
+#define IDENTIFIER 396
 
 
 
@@ -323,7 +329,7 @@ typedef union YYSTYPE
 {
 
 /* Line 1676 of yacc.c  */
-#line 138 "./sparql_parser.y"
+#line 140 "./sparql_parser.y"
 
   raptor_sequence *seq;
   rasqal_variable *variable;
@@ -349,7 +355,7 @@ typedef union YYSTYPE
 
 
 /* Line 1676 of yacc.c  */
-#line 353 "sparql_parser.tab.h"
+#line 359 "sparql_parser.tab.h"
 } YYSTYPE;
 # define YYSTYPE_IS_TRIVIAL 1
 # define yystype YYSTYPE /* obsolescent; will be withdrawn */
