@@ -498,13 +498,23 @@ compare_query_results_compare(compare_query_results* cqr)
       const unsigned char* name;
       rasqal_literal *value1;
       rasqal_literal *value2;
+      int error = 0;
 
       name = rasqal_query_results_get_binding_name(cqr->qr1, bindingi);
 
       value1 = rasqal_query_results_get_binding_value(cqr->qr1, bindingi);
+      /* FIXME */
+      if(value1)
+        rasqal_literal_string_to_native(value1, 0);
       value2 = rasqal_query_results_get_binding_value(cqr->qr2, bindingi);
+      /* FIXME */
+      if(value2)
+        rasqal_literal_string_to_native(value2, 0);
 
-      if(!rasqal_literal_equals(value1, value2)) {
+      /* should have compare as native flag? 
+       * RASQAL_COMPARE_XQUERY doesn't compare all values
+       */
+      if(!rasqal_literal_equals_flags(value1, value2, 0, &error)) {
         /* if different report it */
         raptor_world* raptor_world_ptr;
         void *string;
@@ -665,7 +675,7 @@ main(int argc, char *argv[])
         help = 1;
         break;
 
-      case 'i':
+      case 'l':
         if(optarg) {
           query_language = optarg;
         }
@@ -1047,6 +1057,10 @@ main(int argc, char *argv[])
         rasqal_query_results_rewind(expected_results);
         rasqal_query_results_rewind(results);
 
+        /* FIXME: should NOT do this if results are expected to be ordered */
+        rasqal_query_results_sort(expected_results, rasqal_row_compare);
+        rasqal_query_results_sort(results, rasqal_row_compare);
+
         if(1) {
           compare_query_results* cqr;
           cqr = new_compare_query_results(world,
